import sys
from PyQt5.QtGui import QRegExpValidator
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import json
from enum import Enum
from typing import List


class SetProg(QWidget):
    def __init__(self, parent=None):
        super(SetProg, self).__init__(parent)
        self.tab1 = ProfileWidget("Channel 0")
        self.tab1.setObjectName("Tab 0")
        self.tab2 = ProfileWidget("Channel 1")
        self.tab2.setObjectName("Tab 1")
        self.tab3 = ProfileWidget("Channel 2")
        self.tab3.setObjectName("Tab 2")
        self.tab4 = ProfileWidget("Channel 3")
        self.tab4.setObjectName("Tab 3")

        # Creating a vertical layout to place tabs
        self.rightLayout = QVBoxLayout()
        # Create the main tab widget
        self.mainTabWidget = QTabWidget()
        self.controlTab = QWidget()

        # Adding the main widget to the vertical layout
        self.mainTabWidget.addTab(self.tab1, "Channel 0")
        self.mainTabWidget.addTab(self.tab2, "Channel 1")
        self.mainTabWidget.addTab(self.tab3, "Channel 2")
        self.mainTabWidget.addTab(self.tab4, "Channel 3")

        self.rightLayout.addWidget(self.mainTabWidget)

        # Creating a vertical layout to place control elements
        self.layout = QVBoxLayout()

        # Adding the vertical layout with tabs to the vertical layout
        self.layout.addLayout(self.rightLayout)
        self.setLayout(self.layout)

    # Function for getting a dictionary of profile data from tabs
    def get_main_dict(self):
        profiles_data = {}
        # add dict from all tabs
        for i in range(self.mainTabWidget.count()):
            tab = self.mainTabWidget.widget(i)
            profiles_data[i] = tab.get_dict()
        print(profiles_data)
        return profiles_data


class ProfileWidget(QWidget):
    def __init__(self, name: str, parent=None):
        super(ProfileWidget, self).__init__(parent=parent)
        self.name = name

        # Create the layout for the table and buttons
        layer2 = QHBoxLayout()
        layer3 = QVBoxLayout()

        # Create a table widget and set its properties
        self.tableWidget = QTableWidget(self)
        self.tableWidget.setColumnCount(3)
        self.tableWidget.setRowCount(0)
        self.tableWidget.horizontalHeader().setVisible(False)
        self.tableWidget.setColumnWidth(0, 110)
        self.tableWidget.setColumnWidth(1, 260)
        self.tableWidget.setColumnWidth(2, 120)

        # Create buttons and a combobox
        buttonARM = QPushButton('ARM')
        buttonADD = QPushButton('+')
        buttonSAVE = QPushButton('SAVE')
        buttonLOAD = QPushButton('LOAD')
        buttonDEL1s = QPushButton('-')
        self.typecbox = QComboBox()
        self.typecbox.addItems(["°C", "V"])

        # Create labels for the table columns
        typelabel = QLabel('Value type:')
        labeltime = QLabel()
        labelval = QLabel()
        labeltime.setText('Time')
        labelval.setText("Value")

        # Add widgets to the layout
        layer3.addWidget(self.tableWidget, 0)
        layer2.addWidget(buttonADD, 0)
        layer2.addWidget(buttonDEL1s, 0)
        layer2.addWidget(buttonSAVE, 0)
        layer2.addWidget(buttonLOAD, 0)
        layer2.addWidget(typelabel, 0)
        layer2.addWidget(self.typecbox, 0)
        layer2.addWidget(buttonARM, 0)

        # Set the size of the add and delete buttons
        buttonADD.setFixedSize(60, 25)
        buttonDEL1s.setFixedSize(60, 25)

        # Create the main layout for the widget and add the sub-layouts
        self.layout = QVBoxLayout()
        self.layout.addLayout(layer3)
        self.layout.addLayout(layer2)
        self.setLayout(self.layout)

        # Connect button signals to their corresponding functions

        buttonADD.clicked.connect(lambda: self.add_layer(self.tableWidget.rowCount()))
        buttonDEL1s.clicked.connect(lambda: self.mass_del())
        buttonARM.clicked.connect(lambda: self.parent().parent().parent().get_main_dict())
        buttonSAVE.clicked.connect(lambda: self.saving_procedure())
        buttonLOAD.clicked.connect(lambda: self.table_from_file())
        self.typecbox.currentIndexChanged.connect(self.change_type_value)

    def change_type_value(self):
        for i in range(self.tableWidget.rowCount()):
            row_widget1 = self.tableWidget.cellWidget(i, 1).layout()
            row_widget2 = self.tableWidget.cellWidget(i, 2).layout()
            for k in range(row_widget1.count()):
                if row_widget1.itemAt(k).widget().text() == "°C":

                    row_widget1.itemAt(k).widget().setText('V')
                elif row_widget1.itemAt(k).widget().text() == 'V':

                    row_widget1.itemAt(k).widget().setText("°C")
            for k in range(row_widget2.count()):
                if row_widget2.itemAt(k).widget().text() in ["°C/min", "°C/ms", "°C/s"]:
                    row_widget2.itemAt(k).widget().setText('V/' + row_widget2.itemAt(k).widget().text().split('/')[1])
                elif row_widget2.itemAt(k).widget().text() in ["V/min", "V/ms", "V/s"]:

                    row_widget2.itemAt(k).widget().setText("°C/" + row_widget2.itemAt(k).widget().text().split('/')[1])
            self.tableWidget.cellWidget(i, 1).setLayout(row_widget1)
            self.tableWidget.cellWidget(i, 2).setLayout(row_widget2)

    def saving_procedure(self):
        # Create empty dictionaries to store table data and overall information
        tab_dict = {}
        table_dict = {}

        # Get the unit type from the combo box and add it to the table dictionary
        table_dict['unit'] = self.typecbox.currentText()

        # Check if the table is empty or not
        if self.tableWidget.rowCount() != 0:
            # Loop through each row of the table and create a dictionary for that row's data
            for row in range(self.tableWidget.rowCount()):
                row_dict = {}

                # Get the mode type for this row and add it to the row dictionary
                row_dict['mode_type'] = self.tableWidget.cellWidget(row, 0).text()

                # For each mode type, get the appropriate values from the table and add them to the row dictionary
                if row_dict['mode_type'] == "Isotherm":
                    row_dict['from_value'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(1).widget().text()
                    row_dict['time_value'] = self.tableWidget.cellWidget(row, 2).layout().itemAt(1).widget().text()
                    row_dict['time_type'] = self.tableWidget.cellWidget(row, 2).layout().itemAt(2).widget().text()
                if row_dict['mode_type'] == "Ramp":
                    row_dict['from_value'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(1).widget().text()
                    row_dict['to_value'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(4).widget().text()
                    row_dict['rate_value'] = self.tableWidget.cellWidget(row, 2).layout().itemAt(1).widget().text()
                    row_dict['rate_type'] = self.tableWidget.cellWidget(row, 2).layout().itemAt(2).widget().text()
                    row_dict['to_value'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(4).widget().text()
                if row_dict['mode_type'] == "Sine Segment":
                    row_dict['Ampl'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(1).widget().text()
                    row_dict['Freq'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(4).widget().text()
                    row_dict['Offs'] = self.tableWidget.cellWidget(row, 1).layout().itemAt(6).widget().text()
                    row_dict['time_value'] = self.tableWidget.cellWidget(row, 2).layout().itemAt(1).widget().text()
                    row_dict['time_type'] = self.tableWidget.cellWidget(row, 2).layout().itemAt(2).widget().text()

                    # Add this row's dictionary to the overall table dictionary
                table_dict[row] = row_dict

            # Add the table dictionary to the overall dictionary
            tab_dict['rowdata'] = table_dict

        # Set options for the file dialog
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog

        # Open the file dialog to get the file path to save to
        file_path, _ = QFileDialog.getSaveFileName(self, "New File", "", "JSON Files (*.json)", options=options)

        # If a file path was selected, save the data to that file
        if file_path:
            # If the file path does not end in .json, add it
            if not file_path.endswith('.json'):
                file_path += '.json'

            with open(file_path, 'w') as f:
                json.dump(tab_dict, f, indent=4)

    def table_from_file(self):
        # Define options for file dialog
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog  # add the QFileDialog.DontUseNativeDialog option to options
        # Open file dialog and get file path
        file_path, _ = QFileDialog.getOpenFileName(self, "Open File", "", "JSON Files (*.json)",
                                                   options=options)  # open a file dialog and get the selected file path

        if file_path:
            with open(file_path, 'r') as f:
                data = json.load(f)
        else:
            return

        rows_data = data['rowdata']
        self.tableWidget.clear()
        self.tableWidget.setRowCount(0)
        # Iterate through the keys in rows_data
        row_data = {}
        for key in rows_data.keys():
            if key == 'unit':
                typeval1 = rows_data['unit']
                typeval2 = typeval1

                print(typeval1)
                continue
            else:
                row_data = rows_data[key]

            mode_type = row_data['mode_type']

            if mode_type == "Isotherm":
                val1 = row_data['from_value']
                time = row_data['time_value']
                typetime = row_data['time_type']
            if mode_type == "Ramp":
                val1 = row_data['from_value']
                val2 = row_data['to_value']
                time = row_data['rate_value']
                typetime = row_data['rate_type']
            if mode_type == "Sine Segment":
                val1 = row_data['Ampl']
                val2 = row_data['Freq']
                val3 = row_data['Offs']
                time = row_data['time_value']
                typetime = row_data['time_type']

            row_number = self.tableWidget.rowCount()
            self.tableWidget.insertRow(row_number)
            self.typecbox.setCurrentText(typeval1)

            buttrow = QPushButton(row_data['mode_type'])
            buttrow.setObjectName("but" + str(row_number))
            buttrow.clicked.connect(lambda checked, x=row_number: self.add_layer(x,False))
            self.tableWidget.setCellWidget(row_number, 0, buttrow)

            flabel = QLabel("Ampl: " if mode_type == "Sine Segment" else "From:")
            spineditval1 = QLabel(str(val1))
            labelval1 = QLabel(typeval1 if mode_type != "Sine Segment" else "")
            hlayer1 = QHBoxLayout()
            hlayer1.addWidget(flabel)
            hlayer1.addWidget(spineditval1)
            hlayer1.addWidget(labelval1)
            wid1 = QWidget()
            if mode_type == "Ramp":
                tLabel = QLabel("To: ")
                spineditval2 = QLabel(val2)
                labelval2 = QLabel(typeval2)
                hlayer1.addWidget(tLabel)
                hlayer1.addWidget(spineditval2)
                hlayer1.addWidget(labelval2)
                wid1.setLayout(hlayer1)
            if mode_type == "Sine Segment":
                tLabel = QLabel("Freq: ")
                oLabel = QLabel("Offs: ")
                spineditval2 = QLabel(val2)
                spineditval3 = QLabel(val3)
                labelval2 = QLabel()
                hlayer1.addWidget(tLabel)
                hlayer1.addWidget(spineditval2)
                hlayer1.addWidget(oLabel)
                hlayer1.addWidget(spineditval3)
                hlayer1.addWidget(labelval2)
                wid1.setLayout(hlayer1)
            self.tableWidget.setCellWidget(row_number, 1, wid1)
            wid1.setLayout(hlayer1)

            spineditduration = QLabel(
                "Duration: " if mode_type == "Isotherm" or mode_type == "Sine Segment" else "Rate: ")
            labeltime = QLabel(typetime)
            labelvaltime = QLabel(time)
            hlayer3 = QHBoxLayout()
            hlayer3.addWidget(spineditduration)
            hlayer3.addWidget(labelvaltime)
            hlayer3.addWidget(labeltime)
            wid2 = QWidget()
            wid2.setLayout(hlayer3)
            self.tableWidget.setCellWidget(row_number, 2, wid2)

            # self.typecbox.setEnabled(False)

    def add_layer(self, x: int, b = True):

        dialog = InputDialog1(self.typecbox.currentText())
        self.parent().setWindowModality(Qt.WindowModal)
        if x == self.tableWidget.rowCount():
            row_number = self.tableWidget.rowCount()

        else:

            row_number = x
            mode_type = self.tableWidget.cellWidget(row_number, 0).text()

            if mode_type == "Isotherm":
                from_value = self.tableWidget.cellWidget(row_number, 1).layout().itemAt(1).widget().text()
                time_value = self.tableWidget.cellWidget(row_number, 2).layout().itemAt(1).widget().text()
                time_type = self.tableWidget.cellWidget(row_number, 2).layout().itemAt(2).widget().text()
                dialog.set_values(from_value, time_value, mode_type, time_type)
            if mode_type == "Ramp":
                from_value = self.tableWidget.cellWidget(row_number, 1).layout().itemAt(1).widget().text()
                to_value = self.tableWidget.cellWidget(row_number, 1).layout().itemAt(4).widget().text()
                time_value = self.tableWidget.cellWidget(row_number, 2).layout().itemAt(1).widget().text()
                time_type = self.tableWidget.cellWidget(row_number, 2).layout().itemAt(2).widget().text().split('/')[1]
                dialog.set_values(from_value, time_value, mode_type, time_type, to_value)
            if mode_type == "Sine Segment":
                Ampl = self.tableWidget.cellWidget(row_number, 1).layout().itemAt(1).widget().text()
                Freq = self.tableWidget.cellWidget(row_number, 1).layout().itemAt(4).widget().text()
                Offs = self.tableWidget.cellWidget(row_number, 1).layout().itemAt(6).widget().text()
                time_value = self.tableWidget.cellWidget(row_number, 2).layout().itemAt(1).widget().text()
                time_type = self.tableWidget.cellWidget(row_number, 2).layout().itemAt(2).widget().text()
                dialog.set_values(Ampl, time_value, mode_type, time_type, Freq, Offs)



        if dialog.exec_() == QDialog.Accepted:
            self.tableWidget.insertRow(row_number)
            if b != True:
                self.tableWidget.removeRow(row_number+1)
            mode_type = dialog.type_combo.currentText()
            # Create and set the button widget
            buttrow = QPushButton(mode_type)
            buttrow.setObjectName("but" + str(row_number))
            buttrow.clicked.connect(lambda checked, x=row_number: self.add_layer(x, False))
            self.tableWidget.setCellWidget(row_number, 0, buttrow)
            # Create and set the first value widget
            typeval1 = dialog.val_cb1.text()
            val1 = dialog.edit_val1.text()

            flabel = QLabel("Ampl: " if mode_type == "Sine Segment" else "From:")
            spineditval1 = QLabel(str(val1))
            labelval1 = QLabel(typeval1 if mode_type != "Sine Segment" else "")
            hlayer1 = QHBoxLayout()
            hlayer1.addWidget(flabel)
            hlayer1.addWidget(spineditval1)
            hlayer1.addWidget(labelval1)
            wid1 = QWidget()
            if mode_type == "Ramp":
                typeval2 = dialog.val_cb2.text()
                val2 = dialog.edit_val2.text()

                tLabel = QLabel("To: ")
                spineditval2 = QLabel(val2)
                labelval2 = QLabel(typeval2)
                hlayer1.addWidget(tLabel)
                hlayer1.addWidget(spineditval2)
                hlayer1.addWidget(labelval2)
                wid1.setLayout(hlayer1)
            if mode_type == "Sine Segment":
                typeval2 = ""
                typeval1 = ""
                val2 = dialog.edit_val2.text()
                val3 = dialog.edit_val4.text()

                tLabel = QLabel("Freq: ")
                oLabel = QLabel("Offs: ")
                spineditval2 = QLabel(val2)
                spineditval3 = QLabel(val3)
                labelval2 = QLabel(typeval2)
                hlayer1.addWidget(tLabel)
                hlayer1.addWidget(spineditval2)
                hlayer1.addWidget(oLabel)
                hlayer1.addWidget(spineditval3)
                hlayer1.addWidget(labelval2)
                wid1.setLayout(hlayer1)
            self.tableWidget.setCellWidget(row_number, 1, wid1)
            self.tableWidget.cellDoubleClicked.connect(lambda checked, x=row_number: self.add_layer(x, False))
            wid1.setLayout(hlayer1)
            # Create and set the time widget
            typetime = dialog.val_cb3.currentText()
            time = dialog.edit_val3.text()

            spineditduration = QLabel(
                "Duration: " if mode_type == "Isotherm" or mode_type == "Sine Segment" else "Rate: ")
            labeltime = QLabel(self.typecbox.currentText() + "/" + typetime if mode_type == "Ramp" else typetime)
            labelvaltime = QLabel(time)
            hlayer3 = QHBoxLayout()
            hlayer3.addWidget(spineditduration)
            hlayer3.addWidget(labelvaltime)
            hlayer3.addWidget(labeltime)
            wid2 = QWidget()
            wid2.setLayout(hlayer3)

            self.tableWidget.setCellWidget(row_number, 2, wid2)
            self.tableWidget.cellClicked
        else:
            if b == True:
                self.tableWidget.removeRow(x)

    # self.typecbox.setEnabled(False)



    def mass_del(self):
        # Function to delete selected rows from the table widget
        row_number = self.tableWidget.rowCount()
        # If there is a selection in the table widget, delete the selected rows
        if self.tableWidget.selectionModel().hasSelection():
            # Create an empty list to store the selected indices
            index_list = []

            # Loop through the selected rows and add their indices to the index list
            for model_index in self.tableWidget.selectionModel().selectedRows():
                index = QPersistentModelIndex(model_index)
                index_list.append(index)

            # Loop through the index list and remove the corresponding rows from the table widget
            for index in index_list:
                self.tableWidget.removeRow(index.row())
        # If there is no selection, remove the last row in the table widget
        else:
            self.tableWidget.removeRow(row_number - 1)

        # Loop through the rows of the table widget and update the object names of the buttons
        for i in range(row_number):
            if self.tableWidget.cellWidget(i, 0) is not None:
                self.tableWidget.cellWidget(i, 0).setObjectName("but" + str(i))
                self.tableWidget.cellWidget(i, 0).clicked.disconnect()
                self.tableWidget.cellWidget(i, 0).clicked.connect(lambda checked, x = i : self.add_layer(x, False))

        # If the table widget is now empty, enable the typecbox combo box
        if self.tableWidget.rowCount() == 0:
            self.typecbox.setEnabled(True)

    def get_dict(self):
        # Get the currently selected type from a combo box
        typeval = self.typecbox.currentText()

        # Create an empty dictionary to store data
        data = {}

        # Initialize variables to be used in the loop
        stmoment = 0.
        duration = 0.
        endmoment = 0.
        stval = 0.
        endval = 0.
        ampl = 0.
        freq = 0.
        offs = 0.
        segments = []

        # Iterate through the rows of a table widget
        for i in range(self.tableWidget.rowCount()):
            # Calculate start and end moments for the current segment
            stmoment = stmoment + duration
            endmoment = stmoment + duration

            # Get the type of the current segment
            seg_type = self.tableWidget.cellWidget(i, 0).text()

            # Get the units for the duration of the current segment
            typewid = self.tableWidget.cellWidget(i, 0).text()
            if seg_type == "Ramp":
                typetime = self.tableWidget.cellWidget(i, 2).layout().itemAt(2).widget().text().split("/")[1].strip()
            else:
                typetime = self.tableWidget.cellWidget(i, 2).layout().itemAt(
                    2).widget().text()  # .split("/" if typewid =="Ramp" else "")

            # Convert the duration to seconds based on the units
            if typetime == "s":
                duration = float(duration)
            if typetime == "ms":
                duration = float(duration) / 1000
            if typetime == "min":
                duration = float(duration) * 60

                # Create the appropriate segment based on the segment type
            if seg_type == "Isotherm":
                stval = float(self.tableWidget.cellWidget(i, 1).layout().itemAt(1).widget().text())
                endmoment = stmoment + duration
                segment = IsoSegment(stmoment, endmoment, stval)
                segments.append(segment)

            if seg_type == "Ramp":
                stval = float(self.tableWidget.cellWidget(i, 1).layout().itemAt(1).widget().text())
                endval = float(self.tableWidget.cellWidget(i, 1).layout().itemAt(4).widget().text())
                duration = (endval - stval) / duration
                endmoment = stmoment + duration
                segment = RampSegment(stmoment, endmoment, stval, endval)
                segments.append(segment)

            if seg_type == "Sine Segment":
                ampl = float(self.tableWidget.cellWidget(i, 1).layout().itemAt(1).widget().text())
                freq = float(self.tableWidget.cellWidget(i, 1).layout().itemAt(4).widget().text())
                offs = float(self.tableWidget.cellWidget(i, 1).layout().itemAt(6).widget().text())
                endmoment = stmoment + duration
                segment = SineSegment(stmoment, endmoment, endval, ampl, freq, offs)
                segments.append(segment)

        # Determine the data type based on the selected type in the combo box
        if typeval == "V":
            data_type = DataType.VOLT
        else:
            data_type = DataType.TEMP

        # Create a ProfileData object using the collected data
        data = ProfileData(data_type, segments)

        # Return the created ProfileData object
        return data


class InputDialog1(QDialog):
    def __init__(self, type: str, parent=None):
        QDialog.__init__(self, parent)
        reg_ex = QRegExp("[0-9.]*")

        # create a validator
        validator = QRegExpValidator(reg_ex)

        self.setWindowTitle("Add Option")
        self.val_cb1 = QLabel()
        self.val_cb1.setText(type)
        self.val_cb2 = QLabel()
        self.val_cb2.setText(type)
        self.rate = QLabel()
        self.rate.setText(type + "/")
        self.val_cb3 = QComboBox()
        self.val_cb3.addItems(['s', 'ms', 'min'])
        self.val_cb3.setMinimumSize(80, 25)
        self.horlayout0 = QHBoxLayout()
        self.horlayout1 = QHBoxLayout()
        self.edit_val1 = QLineEdit()
        self.edit_val1.setValidator(validator)
        self.edit_val2 = QLineEdit()
        self.edit_val2.setValidator(validator)
        self.edit_val4 = QLineEdit()  # offset
        self.edit_val4.setValidator(validator)

        self.edit_val3 = QLineEdit()
        self.edit_val3.setValidator(validator)
        self.label1 = QLabel("val1")
        self.label2 = QLabel("val2")
        self.label4 = QLabel("Offset")
        self.label3 = QLabel("Duration")
        self.type_label = QLabel("Choose type")
        self.type_combo = QComboBox()
        self.type_combo.addItems(["Isotherm", "Ramp", "Sine Segment"])
        self.button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.button_box.accepted.connect(self.accept1)
        self.button_box.rejected.connect(self.reject)
        layer1 = QVBoxLayout()
        vlayer1 = QVBoxLayout()
        vlayer2 = QVBoxLayout()
        vlayer3 = QVBoxLayout()
        vlayer4 = QVBoxLayout()
        vlayer5 = QVBoxLayout()
        hlayer1 = QHBoxLayout()
        hlayer2 = QHBoxLayout()
        hlayer3 = QHBoxLayout()

        self.layout = QVBoxLayout()

        vlayer1.addWidget(self.type_label)
        vlayer1.addWidget(self.type_combo)
        vlayer2.addWidget(self.label1)
        hlayer1.addWidget(self.edit_val1)
        hlayer1.addWidget(self.val_cb1)
        vlayer2.addLayout(hlayer1)
        vlayer3.addWidget(self.label2)
        hlayer2.addWidget(self.edit_val2)
        hlayer2.addWidget(self.val_cb2)
        vlayer3.addLayout(hlayer2)
        vlayer4.addWidget(self.label4)
        vlayer4.addWidget(self.edit_val4)
        vlayer5.addWidget(self.label3)
        hlayer3.addWidget(self.edit_val3)
        hlayer3.addWidget(self.rate)
        hlayer3.addWidget(self.val_cb3)
        vlayer5.addLayout(hlayer3)
        self.horlayout0.addLayout(vlayer1)
        self.horlayout0.addLayout(vlayer2)
        self.horlayout0.addLayout(vlayer3)
        self.horlayout0.addLayout(vlayer4)
        self.horlayout0.addLayout(vlayer5)

        self.layout.addLayout(self.horlayout0)

        layer1.addLayout(self.layout)
        layer1.addWidget(self.button_box)
        self.setLayout(layer1)
        self.change_type()
        self.type_combo.currentTextChanged.connect(self.change_type)

    def set_values(self, val1, val3, type_mode, duration_unit, val2='', val4=''):
        index = self.type_combo.findText(type_mode)
        self.type_combo.setCurrentIndex(index)
        self.change_type()

        self.edit_val1.setText(str(val1))
        self.edit_val2.setText(str(val2))
        self.edit_val3.setText(str(val3))
        self.edit_val4.setText(str(val4))

        index2 = self.val_cb3.findText(duration_unit)
        self.val_cb3.setCurrentIndex(index2)

    def text_take(self, type_mode: str, arg: list):
        index = self.type_combo.findText(type_mode)
        self.type_combo.setCurrentIndex(index)
        self.change_type()
        if type_mode == "Isotherm":
            self.edit_val1.setText(arg[1])
            self.edit_val3.setText(arg[2])
            index2 = self.val_cb3.findText(arg[3])
            self.val_cb3.setCurrentIndex(index2)
        if type_mode == "Ramp":
            self.edit_val1.setText(arg[1])
            self.edit_val2.setText(arg[2])
            self.edit_val3.setText(arg[3])
            index2 = self.val_cb3.findText(arg[4])
            self.val_cb3.setCurrentIndex(index2)
        if type_mode == "Sine Segment":
            self.edit_val1.setText(arg[1])
            self.edit_val2.setText(arg[2])
            self.edit_val3.setText(arg[4])
            self.edit_val3.setText(arg[3])
            index2 = self.val_cb3.findText(arg[5])
            self.val_cb3.setCurrentIndex(index2)

    def change_type(self):
        if self.type_combo.currentText() == "Isotherm":
            self.label2.setVisible(False)
            self.val_cb2.setVisible(False)
            self.val_cb1.setVisible(True)
            self.edit_val2.setVisible(False)
            self.edit_val4.setVisible(False)
            self.label4.setVisible(False)
            self.rate.setVisible(False)
            self.label1.setText("Start Value")

            self.label3.setText("Duration")
        if self.type_combo.currentText() == "Ramp":
            self.label2.setVisible(True)
            self.val_cb2.setVisible(True)
            self.val_cb1.setVisible(True)
            self.edit_val2.setVisible(True)
            self.edit_val4.setVisible(False)
            self.label4.setVisible(False)
            self.rate.setVisible(True)
            self.label3.setText("Speed")
            self.label1.setText("Start Value")
            self.label2.setText("Final Value")
        if self.type_combo.currentText() == "Sine Segment":
            self.label2.setVisible(True)
            self.val_cb2.setVisible(True)
            self.edit_val2.setVisible(True)
            self.edit_val4.setVisible(True)
            self.label4.setVisible(True)
            self.rate.setVisible(False)
            self.label3.setText("Duration")
            self.label1.setText("Amplitude")
            self.label2.setText("Frequency")
            self.val_cb1.setVisible(False)
            self.val_cb2.setVisible(False)
            # self.val_cb3.clear()
            # self.val_cb3.addItems(["min", "s", "ms"])

    def accept1(self):
        if self.type_combo.currentText() == "Isotherm":
            if not self.edit_val1.text() or not self.edit_val3.text():
                error_dialog = QMessageBox(QMessageBox.Critical, "Error", "Please enter the values", QMessageBox.Ok,
                                           self)
                error_dialog.exec_()
            else:
                self.accept()

        if self.type_combo.currentText() == "Ramp":
            if not self.edit_val1.text() or not self.edit_val2.text() or not self.edit_val3.text():
                error_dialog = QMessageBox(QMessageBox.Critical, "Error", "Please enter the values", QMessageBox.Ok,
                                           self)
                error_dialog.exec_()
            else:
                self.accept()
        if self.type_combo.currentText() == "Sine Segment":
            if not self.edit_val1.text() or not self.edit_val2.text() or not self.edit_val3.text() or not self.edit_val4.text():
                error_dialog = QMessageBox(QMessageBox.Critical, "Error", "Please enter the values", QMessageBox.Ok,
                                           self)
                error_dialog.exec_()
            else:
                self.accept()


class SegmentType(Enum):
    NONE = 0,
    ISO = 1,
    RAMP = 2,
    SINE = 3


class SegmentData:
    def __init__(self,
                 segment_type: SegmentType,
                 start_time: float,
                 end_time: float,
                 start_value: float,
                 end_value: float):
        self.segment_type = segment_type
        self.start_time = start_time  # in seconds
        self.end_time = end_time  # in seconds
        self.start_value = start_value  # Volts or °C
        self.end_value = end_value  # Volts or °C

    def duration(self) -> float:
        return self.end_time - self.start_time


class IsoSegment(SegmentData):
    def __init__(self,
                 start_time: float,
                 end_time: float,
                 start_value: float):
        super().__init__(SegmentType.ISO, start_time, end_time, start_value, start_value)

    def __repr__(self):
        return f"IsoSegment({self.start_time}, {self.end_time}, {self.start_value})"


class RampSegment(SegmentData):
    def __init__(self,
                 start_time: float,
                 end_time: float,
                 start_value: float,
                 end_value: float):
        super().__init__(SegmentType.RAMP, start_time, end_time, start_value, end_value)

    def __repr__(self):
        return f"RampSegment({self.start_time}, {self.end_time}, {self.start_value}, {self.end_value})"

        # Volts or °C per second

    def rate(self) -> float:
        return (self.end_value - self.start_value) / self.duration()


class SineSegment(SegmentData):
    def __init__(self,
                 start_time: float,
                 end_time: float,
                 start_value: float,
                 amplitude: float,
                 frequency: float,
                 offset: float):
        super().__init__(SegmentType.SINE, start_time, end_time, start_value, start_value)

        self.amplitude = amplitude
        self.frequency = frequency
        self.offset = offset

    def __repr__(self):
        return f"SineSegment({self.start_time}, {self.end_time}, {self.start_value}, {self.end_value}, {self.amplitude}, {self.frequency}, {self.offset}  )"


class DataType(Enum):
    TIME = 1,
    TEMP = 2,
    VOLT = 3


class ProfileData:
    data_type: DataType
    segments: List[SegmentData]

    def __init__(self,
                 data_type: DataType,  # Y-axis
                 segments: List[SegmentData] = None):
        self.data_type = data_type
        if segments is None:
            segments = list()
        self.segments = segments

    def __str__(self):
        result = f"ProfileData(data_type={self.data_type}, segments=["
        for segment in self.segments:
            result += f"\n\t{segment}"
        result += "\n])"
        return result

    def __repr__(self):
        return f"ProfileData({self.data_type}, {self.segments})"


if __name__ == "__main__":
    import sys

    app = QApplication(sys.argv)

    app.setStyle('Fusion')
    example = SetProg()
    example.show()
    sys.exit(app.exec())



import json
import os

import h5py
import numpy as np
import pandas as pd
import requests
from silx.gui import qt
from silx.gui.plot import Plot1D

from pioner.front.calibWindow import *
from pioner.front.configWindow import *
from pioner.front.mainWindowUi import mainWindowUi
from pioner.front.messageWindows import *
from pioner.shared.constants import *
from pioner.shared.settings import FrontSettings


class mainWindow(mainWindowUi):
    def __init__(self, parent=None):
        super(mainWindow, self).__init__(parent)

        self.device = None          # patch for disconnect method. to be changed later
        self.preload_settings()

        self.sysOnButton.clicked.connect(self.sysOnButtonPressed)
        self.sysOffButton.clicked.connect(self.disconnect)
        self.sysDataPathButton.clicked.connect(self.select_data_path)
        self.sysSetupButton.clicked.connect(self.show_help)

        self.calibPathButton.clicked.connect(self.select_calibration_file)
        self.calibViewButton.clicked.connect(self.view_calibraton_info)
        self.calibApplyButton.clicked.connect(self.apply_calib)

        self.applyScanSampleRateButton.clicked.connect(self.apply_sample_rate)
        self.resetScanSampleRateButton.clicked.connect(self.reset_sample_rate)

        self.armButton.clicked.connect(self.fh_arm)
        self.startButton.clicked.connect(self.fh_run)

        self.setTempVoltButton.clicked.connect(self.set_temp_volt)
        self.unsetTempVoltButton.clicked.connect(self.unset_temp_volt)

# for debugging
        self.terror0Button.clicked.connect(self.print_debug)

    def print_debug(self):
        print(self.settings.sample_rate)
        print(self.calibration.comment)
# end for debugging
    
    def sysOnButtonPressed(self):
        self.set_connection()
        # TODO: add continious aquisition after modulation starts to check signals

    def set_connection(self):
        
        if self.sysNoHardware.isChecked() != True:
            # AM: cannot install TANGO on MAC OS. so I've added importing tango here
            # in order to include feature with no-hardware mode 
            try:
                import tango

                self.device = tango.DeviceProxy(self.settings.device_proxy)
                self.device.set_timeout_millis(10000000)
                self.device.set_connection()
            
                if self.settings.calib_path == DEFAULT_CALIBRATION_FILE_REL_PATH:
                    self.apply_default_calib()
                else: 
                    self.apply_calib()
                [item.setEnabled(True) for item in [self.experimentBox, self.controlTab]]

                self.device.set_sample_scan_rate(self.settings.sample_rate)

            except:
                ## No-hardware mode for data processing
                error_text = "No connection to the device or\nTANGO module not found!\nOnly no-hardware mode is possible."
                ErrorWindow(error_text)
                self.run_no_harware()
        else:
            self.run_no_harware()


    
    def disconnect(self):
        if self.device:
           self.device.disconnect()
        [item.setEnabled(False) for item in [self.experimentBox, self.controlTab]]
        self.sysNoHardware.setEnabled(True)
    
    def select_data_path(self):
        dpath = qt.QFileDialog.getExistingDirectory(self, "Choose folder to save experiment files", \
                                                    None, qt.QFileDialog.ShowDirsOnly)
        if dpath: 
            dpath += '/'
            self.settings.data_path = os.path.abspath(dpath)
            self.sysDataPathInput.setText(self.settings.data_path)
        self.sysDataPathInput.setCursorPosition(0)

    def show_help(self):
        self.configWindow = configWindow(parent=self)
        self.configWindow.show()

    def run_no_harware(self):
        self.sysNoHardware.setChecked(True)
        self.controlTabsWidget.setCurrentIndex(1) 
        self.mainTabWidget.setCurrentIndex(1) 
    # ===================================
    # Calibration   

    def select_calibration_file(self):
        fname = qt.QFileDialog.getOpenFileName(self, "Choose calibration file", None, "*.json")[0]
        if fname: 
            self.settings.calib_path = os.path.abspath(fname)
            self.calibPathInput.setText(self.settings.calib_path)
        self.calibPathInput.setCursorPosition(0)

    def apply_calib(self):
        fpath = self.calibPathInput.text()
        if os.path.exists(os.path.abspath(fpath)):
            with open(self.calibPathInput.text(), 'r') as f:
                raw_calib = json.load(f)
                str_calib = json.dumps(raw_calib)
                self.device.load_calibration(str_calib)
                self.device.apply_calibration()
                self.get_calib_from_device()

    def apply_default_calib(self):
        self.device.apply_default_calibration()
        self.get_calib_from_device()
        self.calibPathInput.setText(os.path.abspath(DEFAULT_CALIBRATION_FILE_REL_PATH))
        self.calibPathInput.setCursorPosition(0)

    def get_calib_from_device(self):
        calib_str = self.device.get_current_calibration[1][0]['value']
        calib_dict = json.loads(calib_str)
        self.calibration = Dict2Class(calib_dict)

    def view_calibraton_info(self):
        self.calibWindow = calibWindow(parent=self)
        self.calibWindow.show()
    
    # ===================================
    # Settings
    def preload_settings(self):
        if not os.path.exists(SETTINGS_FOLDER_REL_PATH):
            os.makedirs(SETTINGS_FOLDER_REL_PATH)
        if not os.path.exists(DATA_FOLDER_REL_PATH):
                os.makedirs(DATA_FOLDER_REL_PATH)

        self.load_settings_from_file()
        if not self.settings: 
            quit()

        if not os.path.exists(self.settings.data_path):
            error_text = "Incorrect data path specified.\nIt will be set to {}".format(DATA_FOLDER_REL_PATH)
            ErrorWindow(error_text)
            self.settings.data_path = os.path.abspath(DATA_FOLDER_REL_PATH)
        self.sysDataPathInput.setText(self.settings.data_path)
        self.sysDataPathInput.setCursorPosition(0)

        if not os.path.exists(self.settings.calib_path):
            error_text = "Incorrect calibration file specified.\nIt will be set to default calibration."
            ErrorWindow(error_text)
            self.settings.calib_path = os.path.abspath(DEFAULT_CALIBRATION_FILE_REL_PATH)
        self.calibPathInput.setText(self.settings.calib_path)
        self.calibPathInput.setCursorPosition(0)

        self.scanSampleRateInput.setText(str(self.settings.sample_rate))

        self.freqInput.setText(str(self.settings.modulation_frequency))
        self.amplitudeInput.setText(str(self.settings.modulation_amplitude))
        self.offsetInput.setText(str(self.settings.modulation_offset))  

    def apply_sample_rate(self):
        self.settings.sample_rate = int(self.scanSampleRateInput.text())
        self.device.set_sample_scan_rate(self.settings.sample_rate)
    
    def reset_sample_rate(self):
        self.device.reset_sample_scan_rate()
        self.settings.sample_rate = self.get_sample_rate_from_device()
        self.scanSampleRateInput.setText(str(self.settings.sample_rate))

    def get_sample_rate_from_device(self):    
        return self.device.get_sample_rate[1][0]['value']


    # ===================================
    # Fast heating

    def fh_arm(self):
        xOption = 1000 if self.experimentTimeComboBox.currentIndex()==1 else 1    #0 - time in ms, 1 - time in s
        yOption = self.experimentTempComboBox.currentIndex() #index 0 - temp in C, 1 - volt in V
        uncorrectedProfile = np.array([[],[]], dtype=float)
        for i in range(self.experimentTable.rowCount()):
            uncorrectedProfile = np.hstack((uncorrectedProfile,
                                            [[float(self.experimentTable.item(i, 0).text())],
                                            [float(self.experimentTable.item(i, 1).text())]]))
        correctedProfile = uncorrectedProfile[:, :(np.argmax(uncorrectedProfile[0])+1)]
        correctedProfile = np.insert(correctedProfile, 0, 0, axis=1)
        self.time_table = xOption*correctedProfile[0] # changing s to ms if needed
        self.temp_table = correctedProfile[1]
        
        self.progressBar.setValue(0)  
        self.controlTabsWidget.setCurrentIndex(1) 
        self.progPlot.resultPlot.clear()
        
        y_label = self.experimentTempComboBox.currentText()
        x_label = self.experimentTimeComboBox.currentText()
        self.progPlot.resultPlot.addCurve(x=self.time_table, y=self.temp_table, 
                                            legend="temperature", 
                                            color=self.progPlot.curveColors['red'])
        self.progPlot.resultPlot.getXAxis().setLabel(x_label)
        self.progPlot.resultPlot.getYAxis().setLabel(y_label)

        # generating time_temp_volt_tables as dict, converting it to str
        # to pass to run_fast_heat (tango)
        self.time_temp_volt_tables = {'ch0':{}, 'ch1':{}, 'ch2':{}}

        # 0.1V on 0 channel
        self.time_temp_volt_tables['ch0']['time'] = self.time_table.tolist()
        self.time_temp_volt_tables['ch0']['volt'] = [0.1]*len(self.time_table) 

        # signal to heater - channel 1
        self.time_temp_volt_tables['ch1']['time'] = self.time_table.tolist()
        self.time_temp_volt_tables['ch1']['temp'] = self.temp_table.tolist()

        # 2.5V trigger signal like gate form
        self.time_temp_volt_tables['ch2']['time'] =  self.time_table.tolist()
        self.time_temp_volt_tables['ch2']['time'].insert(-1, self.time_temp_volt_tables['ch2']['time'][-1]-1)
        self.time_temp_volt_tables['ch2']['volt'] = [2.5]*(len(self.time_table)+1)
        self.time_temp_volt_tables['ch2']['volt'][-1] = 0

        self.time_temp_volt_tables_str = json.dumps(self.time_temp_volt_tables)
        self.device.arm_fast_heat(self.time_temp_volt_tables_str)

    def _fh_download_raw_data(self):
        URL = self.settings.http_host+"data/raw_data/raw_data.h5"
        response = requests.get(URL, verify=False)
        with open('./data/raw_data.h5', 'wb') as f:
            f.write(response.content)
    
    def _fh_download_exp_data(self):
        URL = self.settings.http_host+"data/exp_data.h5"
        response = requests.get(URL, verify=False)
        fname = qt.QFileDialog.getSaveFileName(self, "Save data to file", 
                                                self.settings.data_path+'/exp_data.h5', 
                                                "*.h5")[0]
        if fname:
            with open(fname, 'wb') as f:
                f.write(response.content)
        self.currentExpFilePath = fname

    def _fh_transform_exp_data(self):
        self.exp_data = pd.DataFrame({})
        if self.currentExpFilePath:
            with h5py.File(self.currentExpFilePath, 'r') as f:
                data = f['data']
                for key in list(data.keys()):
                    self.exp_data[key] = data[key][:]

    def _fh_plot_data(self):
        self.resultsDataWidget.clear()
        if not self.exp_data.empty:
            _keys = list(self.exp_data.keys())
            _keys.remove('time')
            for idx, key in enumerate(_keys):
                color = self.resultsDataWidget.curveColors[list(self.resultsDataWidget.curveColors.keys())[idx]]
                self.resultsDataWidget.addCurve(self.exp_data['time'], 
                                                self.exp_data[key], 
                                                legend = key,
                                                color=color)
            for curve in self.resultsDataWidget.resultPlot.getItems():
                if curve.getName()!="temp":
                    curve.setVisible(False)
            self.mainTabWidget.setCurrentIndex(1) 
        self.resultsDataWidget.resultPlot.resetZoom()
        

    def fh_run(self):
        self.device.run_fast_heat()
        self._fh_download_raw_data()
        self._fh_download_exp_data()
        self._fh_transform_exp_data()
        self._fh_plot_data()

    # ===================================
    # Iso (set) mode
    def set_temp_volt(self):
        value = float(self.setInput.text())
        print(value)
        key = str(self.setComboBox.currentText())
        if key=="Temperature":
            key="temp"
        if key=="Voltage":
            key="volt"
        print(key)
        chan_temp_volt = {"ch2": {key:value} }
        
        self.chan_temp_volt_str = json.dumps(chan_temp_volt)
        self.device.arm_iso_mode(self.chan_temp_volt_str)
        self.device.run_iso_mode()
    
    def unset_temp_volt(self):
        chan_temp_volt = {"ch1": {"volt":0} }
        self.chan_temp_volt_str = json.dumps(chan_temp_volt)
        self.device.arm_iso_mode(self.chan_temp_volt_str)
        self.device.run_iso_mode()



    # ===================================
    def save_settings_to_file(self, fpath=False):
        # function is used to save settings to default file or to special file if specified
        if not fpath:
            fpath = os.path.abspath(SETTINGS_FILE_REL_PATH)
        else:
            fpath = qt.QFileDialog.getSaveFileName(self, "Select file and path to save settings:", None, "*.json")[0]
        if fpath:
            with open(fpath, 'r') as f:
                file_settings = json.load(f)

            file_settings[SERVER_SETTINGS_FIELD] = self.settings.get_server_settings()
            file_settings[EXPERIMENT_SETTINGS_FIELD] = self.settings.get_exp_settings()

            with open(fpath, 'w') as f:
                json.dump(file_settings, f, separators=(',', ': '), indent=4)

    def load_settings_from_file(self, fpath=False):
        # function is used to load default settings or to load special config from file if specified
        if not fpath:
            fpath = os.path.abspath(SETTINGS_FILE_REL_PATH)
        else:
            fpath = qt.QFileDialog.getOpenFileName(self, "Choose file with settings:", None, "*.json")[0]
        try:
            self.settings = FrontSettings(fpath)
            self.disconnect()
        except:
            error_text = "Settings file is missing or corrupted! Settings will be reset to default."
            ErrorWindow(error_text)
            self.reset_settings()

    def reset_settings(self):
        # reset config params; used default attributes from Params class
        self.settings = FrontSettings(DEFAULT_SETTINGS_FILE_REL_PATH)

    def closeEvent(self, event):
        # dumping current settings to ./settings/settings.json and closing all the windows
        self.save_settings_to_file()
        for window in qt.QApplication.topLevelWidgets():
            window.close()



# Turns a dictionary into a class
class Dict2Class(object):
    def __init__(self, my_dict):
        for key in my_dict:
            setattr(self, key, my_dict[key])
        self.my_dict = my_dict
    def get_dict(self):
        return self.my_dict




import json
import logging
import os
import uldaq as ul
from tango.server import AttrWriteType, Device, attribute, command, pipe

from shared.constants import *
from shared.calibration import Calibration
from shared.settings import BackSettings
from back.daq_device import DaqDeviceHandler
from back.fastheat import FastHeat
from back.iso_mode import IsoMode



class NanoControl(Device):
    _fh: FastHeat

    def init_device(self):
        Device.init_device(self)
        self._do_initial_setup()

    def _do_initial_setup(self):
        if not (os.path.exists(LOGS_FOLDER_REL_PATH)):
            os.makedirs(LOGS_FOLDER_REL_PATH)
        if not (os.path.exists(RAW_DATA_FOLDER_REL_PATH)):
            os.makedirs(RAW_DATA_FOLDER_REL_PATH)

        logging.basicConfig(filename=NANOCONTROL_LOG_FILE_REL_PATH, encoding='utf-8', level=logging.DEBUG,
                            filemode="w", format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %H:%M:%S')  # TODO: remove from class

        self._calibration = Calibration()
        self.apply_default_calibration()
        self._time_temp_table = dict(time=[], temperature=[])

        self._settings = BackSettings(SETTINGS_FILE_REL_PATH)
        daq_params = self._settings.daq_params
        self._daq_device_handler = DaqDeviceHandler(daq_params)
        logging.info('TANGO: Initial setup done.')

    @command
    def set_connection(self):
        try:
            self._daq_device_handler.try_connect()
            logging.info('TANGO: Successfully connected.')
        except ul.ULException as e:
            logging.error("TANGO: ERROR. ULException while setting connection."
                          "Code: {}, message: {}.".format(e.error_code, e.error_message))
            self._daq_device_handler.quit()
        except TimeoutError as e:
            logging.error("TANGO: ERROR. Timeout exception while setting connection: {}".format(e))
            self._daq_device_handler.quit()

    @command
    def disconnect(self):
        self._daq_device_handler.disconnect()
        logging.info('TANGO: Successfully disconnected.')

    @command
    def log_device_info(self):
        try:
            descriptor = self._daq_device_handler.get_descriptor()
            logging.info("TANGO: Product info: {}".format(descriptor.dev_string))
            logging.info("TANGO: Interface type: {}".format(descriptor.dev_interface))
        except ul.ULException as e:
            logging.error("TANGO: ERROR. Exception while logging info."
                          "Code: {}, message: {}.".format(e.error_code, e.error_message))

    @pipe
    def get_info(self):
        return ('Information',
                dict(developer='Alexey Melnikov & Evgenii Komov',
                     contact='alexey0703@esrf.fr',
                     model='nanocal 2.0',
                     version_number=0.1
                     )
                )

    # ===================================
    # Calibration

    @command(dtype_in=str)
    def load_calibration(self, str_calib):
        with open(CALIBRATION_FILE, 'w') as f:
            json.dump(json.loads(str_calib), f, separators=(',', ': '), indent=4)
            logging.info('TANGO: Calibration file {} was updated from external file.'.format(CALIBRATION_FILE))

    @command
    def apply_default_calibration(self):
        try:
            self._calibration.read(DEFAULT_CALIBRATION_FILE)
            logging.info('TANGO: Calibration was applied from {}'.format(DEFAULT_CALIBRATION_FILE))
        except Exception as e:
            logging.error("TANGO: Error while applying default calibration: {}.".format(e))

    @command
    def apply_calibration(self):
        try:
            self._calibration.read(CALIBRATION_FILE)
            logging.info('TANGO: Calibration was applied from {}'.format(CALIBRATION_FILE))
        except Exception as e:
            logging.error("TANGO: ERROR. Exception while applying calibration: {}.".format(e))

    @pipe(label="Current calibration")
    def get_current_calibration(self):
        self.__calib_dict = 'calib', dict(calib=self._calibration.get_str())
        return self.__calib_dict

    # ===================================
    # Settings
    @pipe(label="Current sample rate")
    def get_sample_rate(self):
        self.__sr = 'sr', dict(sr=self._settings.ai_params.sample_rate)
        return self.__sr

    @command(dtype_in=int)
    def set_sample_scan_rate(self, scan_rate):
        self._settings.ai_params.sample_rate = scan_rate
        self._settings.ao_params.sample_rate = scan_rate
        logging.info("TANGO: Sample scan rate changed to: {}".format(scan_rate))
    
    @command
    def reset_sample_scan_rate(self):
        self._settings.parse_ai_params()
        self._settings.parse_ao_params()

    # ===================================
    # Fast heating

    @command(dtype_in=str)
    def arm_fast_heat(self, time_temp_volt_tables_str):
        self._time_temp_volt_tables = json.loads(time_temp_volt_tables_str)
        self._fh = FastHeat(self._daq_device_handler, self._settings,
                            self._time_temp_volt_tables, self._calibration,
                            ai_channels = [0,1,3,4,5],
                            FAST_HEAT_CUSTOM_FLAG=False)
        self._fh.arm()
        logging.info("TANGO: Fast heating armed.")

    @command
    def run_fast_heat(self):
        if self._fh.is_armed():
            logging.info("TANGO: Fast heating started.")
            self._fh.run()
            
            logging.info("TANGO: Fast heating finished.")
        else:
            logging.warning("TANGO: WARNING. Fast heating cannot be started, since it should be armed first.")


    # ===================================
    # Iso (set) mode
    
    @command(dtype_in=str)
    def arm_iso_mode(self, chan_temp_volt_str):
        self._chan_temp_volt = json.loads(chan_temp_volt_str)
        self._im = IsoMode(self._daq_device_handler, self._settings,
                            self._chan_temp_volt, self._calibration)
        channel, voltage = self._im.arm()
        logging.info("TANGO: Static (iso) mode armed at channel {} with {} V.".format(channel, voltage))

    @command
    def run_iso_mode(self):
        if self._im.is_armed():
            self._im.run(do_ai=True)
        else:
            logging.warning("TANGO: WARNING. Static (iso) mode cannot be started, since it should be armed first.")
        

    @command 
    def stop_ai(self):
        # self._im.ai_stop()
        logging.info("TANGO: test")
        # self._daq_device_handler.get_ai_device.scan_stop()
        

if __name__ == '__main__':
    NanoControl.run_server()



import glob
import logging
import os
from ctypes import Array
from typing import List
import pandas as pd
import uldaq as ul

from shared.constants import *
from shared.settings import BackSettings
from back.ai_device import AiDeviceHandler
from back.ao_data_generators import ScanDataGenerator
from back.ao_device import AoDeviceHandler
from back.daq_device import DaqDeviceHandler

# TODO: check why we create analog devices inside scanning methods


class ExperimentManager:
    _ai_device_handler: AiDeviceHandler
    _ao_device_handler: AoDeviceHandler
    _ao_buffer: Array[float]

    def __init__(self, daq_device_handler: DaqDeviceHandler,
                 settings: BackSettings):
        self._daq_device_handler = daq_device_handler
        self._ai_params = settings.ai_params
        self._ao_params = settings.ao_params

        ExperimentManager._do_smth_strange()  # TODO: check and try to avoid this action

    @staticmethod
    def _do_smth_strange():
        # Strange, but the first invoke of pandas.to_hdf takes a lot of time.
        # So in order not to lose points during acquisition, we invoke it here with empty dataframe
        df = pd.DataFrame([])
        df.to_hdf(RAW_DATA_FILE_REL_PATH, key='dataset', format='table', append=True, mode='a')

        # before starting, removing the previous generated files with data from separated buffers
        h5_files_to_remove_regex = RAW_DATA_FOLDER_REL_PATH + '/' + RAW_DATA_BUFFER_FILE_PREFIX + "*.h5"

        h5_files = glob.glob(h5_files_to_remove_regex, recursive=True)
        h5_files.append(RAW_DATA_FILE_REL_PATH)
        for file in h5_files:
            try:
                os.remove(file)
            except:
                pass

    def get_ai_data(self) -> pd.DataFrame:
        df = pd.DataFrame(pd.read_hdf(RAW_DATA_FILE_REL_PATH, key='dataset'))
        return df

    def _transform_ai_data(self, ai_channels: list[int], df) -> pd.DataFrame:
        channels_num = self._ai_params.high_channel - self._ai_params.low_channel + 1
        one_chan_len = int(len(df) / channels_num)
        multi_index = pd.MultiIndex.from_product([list(range(one_chan_len)), list(range(channels_num))])
        df.index = multi_index
        df = df.unstack()
        df.columns = df.columns.droplevel()
        df = df[ai_channels]
        return df

    # for limited scans (one AO buffer will be applied)
    def ao_scan(self, voltage_profiles):
        logging.info("EXPERIMENT_MANAGER: AO SCAN mode. Wait until scan is finished.\n")
        self._ao_params.options = ul.ScanOption.BLOCKIO  # 2
        
        self._ao_buffer = ScanDataGenerator(voltage_profiles,
                                            self._ao_params.low_channel,
                                            self._ao_params.high_channel).get_buffer()

        self._ao_device_handler = AoDeviceHandler(self._daq_device_handler.get_ao_device(),
                                                  self._ao_params)
        # need to stop AO before scan
        if self._ao_device_handler.status == ul.ScanStatus.RUNNING:
            self._ao_device_handler.stop()

        # TODO: only for testing of digital triggering. 
        # put later in separate class

        ##################################################################

        self._ao_device_handler.scan(self._ao_buffer)

    # for setting voltage
    def ao_set(self, ao_channel, voltage):
        logging.info("EXPERIMENT_MANAGER: AO STATIC (SET) mode.\n")
        self._ao_params.options = ul.ScanOption.DEFAULTIO  # 0
        self._ao_device_handler = AoDeviceHandler(self._daq_device_handler.get_ao_device(),
                                                  self._ao_params)
        # # need to stop AO before scan
        if self._ao_device_handler.status == ul.ScanStatus.RUNNING:
            self._ao_device_handler.stop()
        
        self._ao_device_handler.iso_mode(ao_channel, voltage)


    def ai_continuous(self, ai_channels: list[int], do_save_data: bool):
        # AI buffer is 1 s and AI is made in loop. AO buffer equals to AO profile length.
        # if do_save_data: svae all in separate buffers (for finit ai/ao scan)
        # else: dump into dummy buffer file (for endless ai scan)
        
        self._ai_params.options = ul.ScanOption.CONTINUOUS  # 8
        self._ai_device_handler = AiDeviceHandler(self._daq_device_handler.get_ai_device(),
                                                  self._ai_params)

        # need to stop acquisition before scan
        if self._ai_device_handler.status == ul.ScanStatus.RUNNING:
            self._ai_device_handler.stop()

        self._ai_device_handler.scan()
        self._read_data_loop(do_save_data)
        
        if do_save_data:
            df = pd.DataFrame(pd.read_hdf(RAW_DATA_FILE_REL_PATH, key='dataset'))
            df = self._transform_ai_data(ai_channels, df)
            df.to_hdf(RAW_DATA_FILE_REL_PATH, key='dataset', format='table', mode='w')

        logging.info('EXPERIMENT_MANAGER: Continuous AI finished.')

    def ai_continuous_stop(self):
        self._ai_device_handler.stop()
        
    def _read_data_loop(self, do_save_data: bool):
        # if do_save_data: svae all in separate buffers (for finit ai/ao scan)
        # else: dump into dummy buffer file (for endless ai scan)
        try:
            tmp_ai_data = self._ai_device_handler.get_buffer()

            is_buffer_high_half = True
            half_buffer_len = int(len(tmp_ai_data) / 2)
            buffer_index = 0

            channels_num = self._ao_params.high_channel - self._ao_params.low_channel + 1
            if do_save_data:
                buffers_num = int(len(self._ao_buffer) / (self._ao_params.sample_rate * channels_num))
                logging.info('EXPERIMENT_MANAGER: Finite ai scan with data saving started.')
            else:
                buffers_num = 1 # just big number for quazi-infinite loop 
                # TODO: change later to separate thread process in order to be able stop the scan
                # with tango command
                logging.info('EXPERIMENT_MANAGER: Infinite ai scan with data overwriting started.')

            if not os.path.exists(RAW_DATA_FOLDER_REL_PATH):
                os.makedirs(RAW_DATA_FOLDER_REL_PATH)

            while True:
                try:
                    # Get AI operation status and index
                    ai_status, ai_transfer_status = self._ai_device_handler.status()
                    ai_index = ai_transfer_status.current_index

                    if buffer_index >= buffers_num:
                        self._ai_device_handler.stop()
                        if do_save_data:
                            # merging all the buffer files into one file raw_data.h5
                            fpath = RAW_DATA_FILE_REL_PATH
                            for i in list(range(buffers_num)):
                                buf_path = os.path.join(RAW_DATA_FOLDER_REL_PATH, RAW_DATA_BUFFER_FILE_FORMAT.format(i))
                                df = pd.DataFrame(pd.read_hdf(buf_path, key='dataset'))
                                df.to_hdf(RAW_DATA_FILE_REL_PATH, key='dataset', format='table', append=True, mode='a')
                        break  

                    if ai_index > half_buffer_len and is_buffer_high_half and ai_status==1:
                        # reading low half 
                        df = pd.DataFrame(tmp_ai_data[:half_buffer_len])
                        if do_save_data:
                            logging.info('EXPERIMENT_MANAGER: Reading low half. Index = {}. Buffer index = {}'.format(ai_index, buffer_index))
                            fpath = os.path.join(RAW_DATA_FOLDER_REL_PATH, RAW_DATA_BUFFER_FILE_FORMAT.format(buffer_index))
                            df.to_hdf(fpath, key='dataset', format='table', append=True, mode='a')
                        else:
                            logging.info('EXPERIMENT_MANAGER: read')
                            fpath = os.path.join(RAW_DATA_FOLDER_REL_PATH, BUFFER_DUMMY_1)
                            df.to_hdf(fpath, key='dataset', format='table', append=False, mode='a')
                        is_buffer_high_half = False
                    elif ai_index < half_buffer_len and not is_buffer_high_half and ai_status==1:
                        # reading high half
                        df = pd.DataFrame(tmp_ai_data[half_buffer_len:])
                        if do_save_data:
                            logging.info('EXPERIMENT_MANAGER: Reading high half. Index = {}. Buffer index = {}'.format(ai_index, buffer_index))
                            fpath = os.path.join(RAW_DATA_FOLDER_REL_PATH, RAW_DATA_BUFFER_FILE_FORMAT.format(buffer_index))
                            df.to_hdf(fpath, key='dataset', format='table', append=True, mode='a')
                        else:
                            logging.info('EXPERIMENT_MANAGER: read')
                            fpath = os.path.join(RAW_DATA_FOLDER_REL_PATH, BUFFER_DUMMY_2)
                            df.to_hdf(fpath, key='dataset', format='table', append=False, mode='a')
                        is_buffer_high_half = True
                        buffer_index += 1
                except (ValueError, NameError, SyntaxError):
                    break

        except KeyboardInterrupt:
            logging.warning('EXPERIMENT_MANAGER: WARNING. Acquisition aborted.')
            pass



    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        if exc_value is not None:
            logging.error("EXPERIMENT_MANAGER: ERROR. Exception {} of type {}. Traceback: {}".format(exc_value, exc_type, exc_tb))

        # if self._daq_device_handler:
        #     if self._ai_device_handler:
        #         if self._ai_device_handler.status() == ul.ScanStatus.RUNNING:
        #             self._ai_device_handler.stop()
        #         if self._ao_device_handler.status() == ul.ScanStatus.RUNNING:
        #             self._ao_device_handler.stop()
            # self._daq_device_handler.quit()
        # TODO: maybe add here dumping into h5 file??  # @EK: seems quite reasonable



from silx.gui import qt

class ErrorWindow(qt.QMessageBox):
    def __init__(self, error_text: str, parent=None):
        super(ErrorWindow, self).__init__(parent)
        self.setText(error_text)
        self.setWindowTitle("Error")
        self.setIcon(qt.QMessageBox.Critical)
        self.addButton(qt.QMessageBox.Ok)
        self.exec()

class MessageWindow(qt.QMessageBox):
    def __init__(self, message_text: str, parent=None):
        super(MessageWindow, self).__init__(parent)
        self.setText(message_text)
        self.setWindowTitle("Sorry...")
        self.setIcon(qt.QMessageBox.Information)
        self.addButton(qt.QMessageBox.Ok)
        self.exec()

class YesCancelWindow(qt.QMessageBox):
    def __init__(self, message_text: str, parent=None):
        super(YesCancelWindow, self).__init__(parent)
        self.setText(message_text)
        self.setWindowTitle("Warning")
        self.setIcon(qt.QMessageBox.Warning)
        self.addButton(qt.QMessageBox.Yes)
        self.addButton(qt.QMessageBox.Cancel)


from silx.gui import qt

from pioner.shared.constants import *
from pioner.shared.settings import *
from pioner.front.messageWindows import *


class configWindow(qt.QDialog):
    def __init__(self, parent=None):
        super(configWindow, self).__init__(parent=parent)

        # ####### UI setup
        # ########################################
        self.setWindowTitle("Help & Configuration")
        self.setFixedHeight(400)
        self.setFixedWidth(300)
        self.setWindowFlag(qt.Qt.WindowContextHelpButtonHint, False)

        mainLayout = qt.QVBoxLayout()
        mainLayout.setAlignment(qt.Qt.AlignHCenter)
        self.setLayout(mainLayout)

        labl = qt.QLabel("Some help will be here :)")
        labl.setAlignment(qt.Qt.AlignHCenter)
        mainLayout.addWidget(labl)
        labl = qt.QLabel("Nanocontrol v.0.1")
        labl.setAlignment(qt.Qt.AlignHCenter)
        labl.setFont(qt.QFont("Times", weight=qt.QFont.Bold))
        mainLayout.addWidget(labl)
        labl = qt.QLabel("Melnikov Alexey & Komov Evgenii")
        labl.setAlignment(qt.Qt.AlignHCenter)
        mainLayout.addWidget(labl)
        labl = qt.QLabel(u"<p><a href='"'mailto:alexey.melnikov@esrf.fr'"'>alexey.melnikov@esrf.fr</a>  </p>")
        labl.setAlignment(qt.Qt.AlignHCenter)
        mainLayout.addWidget(labl)

        ######## Configuration parameters
        mainLayout.addStretch()
        mainLayout.addStretch()
        self.configGroupBox = qt.QGroupBox()
        mainLayout.addWidget(self.configGroupBox)

        lout_0 = qt.QVBoxLayout()
        lout_0.setSpacing(2)
        self.configGroupBox.setLayout(lout_0)
        labl = qt.QLabel("Configuration parameters")
        labl.setAlignment(qt.Qt.AlignHCenter)
        labl.setFont(qt.QFont("Times", weight=qt.QFont.Bold))
        lout_0.addWidget(labl)
        lout_0.addSpacing(10)

        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        labl = qt.QLabel("Tango host: ")
        labl.setMinimumWidth(75)
        lout_1.addWidget(labl)
        self.tangoHostInput = qt.QLineEdit()
        self.tangoHostInput.setFrame(False)
        self.tangoHostInput.setText(self.parent().settings.tango_host)
        lout_1.addWidget(self.tangoHostInput)

        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        labl = qt.QLabel("Device proxy: ")
        labl.setMinimumWidth(75)
        lout_1.addWidget(labl)
        self.deviceProxyInput = qt.QLineEdit()
        self.deviceProxyInput.setFrame(False)
        self.deviceProxyInput.setText(self.parent().settings.device_proxy)
        lout_1.addWidget(self.deviceProxyInput)

        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        labl = qt.QLabel("HTTP host: ")
        labl.setMinimumWidth(75)
        lout_1.addWidget(labl)
        self.httpHostInput = qt.QLineEdit()
        self.httpHostInput.setFrame(False)
        self.httpHostInput.setText(self.parent().settings.http_host)
        lout_1.addWidget(self.httpHostInput)

        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        lout_1.setSpacing(1)
        self.applyConfigButton = qt.QPushButton("Apply")
        self.applyConfigButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_1.addWidget(self.applyConfigButton)
        self.loadConfigButton = qt.QPushButton("Load")
        self.loadConfigButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_1.addWidget(self.loadConfigButton)
        self.saveConfigButton = qt.QPushButton("Save")
        self.saveConfigButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_1.addWidget(self.saveConfigButton)
        self.resetConfigButton = qt.QPushButton("Reset")
        self.resetConfigButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_1.addWidget(self.resetConfigButton)

        if parent.sysNoHardware.isChecked() == True:
            self.configGroupBox.setEnabled(False)
            self.httpHostInput.setText("no-hardware mode")
            self.deviceProxyInput.setText("no-hardware mode")
            self.tangoHostInput.setText("no-hardware mode")

        # ####### end of UI setup
        # ########################################
        self.applyConfigButton.clicked.connect(self.apply_settings)
        self.saveConfigButton.clicked.connect(self.save_settings_to_file)
        self.loadConfigButton.clicked.connect(self.load_settings_from_file)
        self.resetConfigButton.clicked.connect(self.reset_settings)

    def apply_settings(self):
        self.parent().settings.tango_host = self.tangoHostInput.text()
        self.parent().settings.device_proxy = self.deviceProxyInput.text()
        self.parent().settings.http_host = self.httpHostInput.text()

    def load_settings_from_file(self):
        self.parent().load_settings_from_file(fpath=True)
        self.tangoHostInput.setText(self.parent().settings.tango_host)
        self.deviceProxyInput.setText(self.parent().settings.device_proxy)
        self.httpHostInput.setText(self.parent().settings.http_host)

    def save_settings_to_file(self):
        self.parent().save_settings_to_file(fpath=True)
    
    def reset_settings(self):
        self.parent().reset_settings()
        self.tangoHostInput.setText(self.parent().settings.tango_host)
        self.deviceProxyInput.setText(self.parent().settings.device_proxy)
        self.httpHostInput.setText(self.parent().settings.http_host)

if __name__ == "__main__":
    import sys
    app = qt.QApplication(sys.argv)
    example = configWindow()
    example.show()
    sys.exit(app.exec())


import logging
from typing import Dict
import h5py
import numpy as np
import pandas as pd
from scipy import interpolate

from shared.constants import *
from shared.settings import BackSettings
from shared.utils import temperature_to_voltage
from shared.calibration import Calibration
from back.daq_device import DaqDeviceHandler
from back.experiment_manager import ExperimentManager


class FastHeat:
    # receives time_temp_volt_tables as dict :
    # {"ch0": {"time":[list], "temp":[list]}, 
    # "ch1": {"time":[list], "temp":[list]},
    # "ch2": {"time":[list], "temp":[list]}, 
    # "ch3": {"time":[list], "temp":[list]}, }
    # voltage can be used instead of temperature ("volt" instead of "temp")
    # if "volt" - no calibration applied
    # if "temp" - calibration applied
    # raw_ai_data transfroms to exp_ai_data with respect to calibration
    # raw_ai_data without calibration transformation can be accessed from RAW_DATA_FILE_REL_PATH
    # only data from specified ai_channels will be saved. For normal fast heating it is [0,1,3,4,5]

    def __init__(self, daq_device_handler: DaqDeviceHandler,
                 settings: BackSettings,
                 time_temp_volt_tables: dict,
                 calibration: Calibration,
                 ai_channels: list[int],
                 FAST_HEAT_CUSTOM_FLAG=False):

        self._daq_device_handler = daq_device_handler
        self._settings = settings
        self._time_temp_volt_tables = time_temp_volt_tables
        self._calibration = calibration
        self._ai_channels = ai_channels
        self._FAST_HEAT_CUSTOM_FLAG = FAST_HEAT_CUSTOM_FLAG

        self._voltage_profiles = dict()
        [self._profile_time, self._samples_per_channel] = self._check_time_temp_volt_tables(self._time_temp_volt_tables)


    def arm(self):
        for chan, table in self._time_temp_volt_tables.items():
            key = list(table.keys())
            key.remove('time')
            key = key[0]

            self._voltage_profiles[chan] = self._interpolate_profile(table['time'], table[key])
            if key=='temp':
                self._voltage_profiles[chan] = temperature_to_voltage(self._voltage_profiles[chan], self._calibration)

    def is_armed(self) -> bool:
        return bool(self._voltage_profiles)

    def run(self):
        with ExperimentManager(self._daq_device_handler,
                               self._settings) as em:
            em.ao_scan(self._voltage_profiles)
            em.ai_continuous(self._ai_channels, do_save_data=True)
            self._ai_data = em.get_ai_data()  # TODO: check warning

        if not self._FAST_HEAT_CUSTOM_FLAG:
            self._apply_calibration()
        self._save_data()
        self._add_info_to_file()

    def _check_time_temp_volt_tables(self, time_temp_volt_tables):
        # 1) checks if all the channels profiles have the same duration
        # 2) for each channel checks if time and corresponding temp/volt 
        # profile have the same length
        # 3) checks if specified time profile can be packed into 1 s buffers

        time_lengths = []
        max_len = 0
        for chan, table in time_temp_volt_tables.items():
            key = list(table.keys())
            key.remove('time')
            key = key[0]
            
            if len(table['time']) != len(table[key]):
                error_str = "Different input number of points for time and temperature/voltage at channel " + chan
                logging.error('FAST_HEAT: ', error_str)
                raise ValueError(error_str)
            
            time_lengths.append(table['time'][-1])
            time_lengths = list(set(time_lengths))
        
        if len(time_lengths)>1:
            error_str = "Different duration for channels profiles"
            logging.error('FAST_HEAT: ', error_str)
            raise ValueError(error_str)

        if time_lengths[-1] % 1000. != 0:
            error_str = "Input profile time cannot be packed into integer buffers (1000 ms)."  # TODO: check
            logging.error('FAST_HEAT: ', error_str)
            raise ValueError(error_str)
        else:
            sample_rate = self._settings.ao_params.sample_rate
            samples_per_channel = int(time_lengths[-1] / 1000. * sample_rate)
        # if tests are fine, returns the duration of profile, the same for each channel
        # + amount of samples (buffer length in points) for each profile
        return time_lengths[0], samples_per_channel

    def _interpolate_profile(self, profile_time, profile_temp_or_volt) -> np.array:
        # profile interpolation with respect to time column in table
        interpolation = interpolate.interp1d(x=profile_time, y=profile_temp_or_volt, kind='linear')
        
        time_program_points = np.linspace(profile_time[0], profile_time[-1], self._samples_per_channel)
        temp_or_volt_program_points = interpolation(time_program_points)

        return temp_or_volt_program_points

    def _apply_calibration(self):

        # add timescale in ms
        end_time_ms = 1000*len(self._ai_data)/self._settings.ao_params.sample_rate
        step = 1000/self._settings.ao_params.sample_rate
        self._ai_data['time'] = np.arange(0, end_time_ms, step)

        # Taux - mean for the whole buffer
        Uaux = self._ai_data[3].mean()
        Taux = 100. * Uaux
        if Taux < -12.:  # correction for AD595 below -12 C
            Taux = 2.6843 + 1.2709 * Taux + 0.0042867 * Taux * Taux + 3.4944e-05 * Taux * Taux * Taux
        self._ai_data['Taux'] = Taux
        
        # Utpl or temp - temperature of the calibrated internal thermopile + Taux
        self._ai_data[4] *= (1000. / 11.)  # scaling to mV with the respect of amplification factor of 11
        ax = self._ai_data[4] + self._calibration.utpl0
        self._ai_data['temp'] = self._calibration.ttpl0 * ax + self._calibration.ttpl1 * (ax ** 2)
        self._ai_data['temp'] += Taux

        # temp-hr ??? add explanation Umod mV
        
        self._ai_data[1] *= (1000. / 121.)  # scaling to mV; why 121?? amplifier cascade??
        ax = self._ai_data[1] + self._calibration.utpl0
        self._ai_data['temp-hr'] = self._calibration.ttpl0 * ax + self._calibration.ttpl1 * (ax ** 2)

        # Uref
        # ===================
        # profile = pd.DataFrame(self.voltage_profiles['ch1'])
        # Uref = pd.concat(profile*(int(len(self.ai_data[0])/len(profile))), ignore_index=True) # generating repeated profiles
        # self.ai_data['Uref'] = profile
        
        # Thtr
        self._ai_data[5] *= 1000.  # Uhtr mV
        Rhtr = self._ai_data[5] * 0.
        Ih = self._calibration.ihtr0 + self._ai_data[0] * self._calibration.ihtr1
        Rhtr.loc[Ih!=0] = (self._ai_data[5] - self._ai_data[0] * 1000. + self._calibration.uhtr0) * self._calibration.uhtr1 / Ih
        # Rhtr.loc[Ih==0] = 0
        Thtr = self._calibration.thtr0 + \
               self._calibration.thtr1 * (Rhtr + self._calibration.thtrcorr) + \
               self._calibration.thtr2 * ((Rhtr + self._calibration.thtrcorr) ** 2)
        self._ai_data['Thtr'] = Thtr
        self._ai_data['Uref'] = self._voltage_profiles['ch1']   # Uref equals to the voltage sent on guard heaters
        
        self._ai_data.drop(self._ai_channels, axis=1, inplace=True)

    def _save_data(self):
        fpath = EXP_DATA_FILE_REL_PATH
        with h5py.File(fpath, 'w') as f:
            data = f.create_group('data')
            data.create_dataset('time', data=self._ai_data['time'])
            data.create_dataset('Taux', data=self._ai_data['Taux'])
            data.create_dataset('Thtr', data=self._ai_data['Thtr'])
            data.create_dataset('Uref', data=self._ai_data['Uref'])
            data.create_dataset('temp', data=self._ai_data['temp'])
            data.create_dataset('temp-hr', data=self._ai_data['temp-hr'])

    def _add_info_to_file(self):
        fpath = EXP_DATA_FILE_REL_PATH
        with h5py.File(fpath, 'a') as f:
            f.create_dataset('calibration', data=self._calibration.get_str())
            f.create_dataset('settings', data=self._settings.get_str())
            
            programs = f.create_group('temp_volt_programs')
            for chan, table in self._time_temp_volt_tables.items():
                key = list(table.keys())
                key.remove('time')
                key = key[0]          # get volt or temp key

                program = programs.create_group(chan)
                program.create_dataset('time', data=table['time'])
                program.create_dataset(key, data=table[key])

            profiles = f.create_group('voltage_profiles')
            for chan in self._voltage_profiles.keys():
                profiles.create_dataset(chan, data=self._voltage_profiles[chan])

if __name__ == '__main__':
    
    settings = BackSettings(SETTINGS_FILE_REL_PATH)
    time_temp_volt_tables = {'ch0':{'time':[0, 3000], 'volt':[1,1]},
                            'ch1':{'time':[0, 100, 1100, 1900, 2900, 3000], 'temp':[0, 0, 5, 5, 0, 0]},
                            'ch2':{'time':[0, 3000], 'volt':[5,5]},
                            # 'ch3':{'time':[0,2000,2000], 'volt':[0,0,0]}
                            }
    calibration = Calibration()
    ai_channels = [0, 1, 3, 4, 5]

    daq_params = settings.daq_params
    daq_device_handler = DaqDeviceHandler(daq_params)
    daq_device_handler.try_connect()

    fh = FastHeat(daq_device_handler, settings, time_temp_volt_tables, calibration, ai_channels)
    fh.arm()
    fh.run()

    daq_device_handler.disconnect()



from messageWindows import *

# AM: check later if this class is indeed nessesary. Now almost all
# buttons are disabled after no-hardware mode switched on.

class virtualDevice:
    def __init__(self):
        self.error_text = "Impossible action.\nNo-hardware mode is active"
        
    def disconnect(self):
        pass

    def load_calibration(self):
        ErrorWindow(self.error_text)
    
    def apply_calibration(self):
        ErrorWindow(self.error_text)

    def apply_default_calibration(self):
        ErrorWindow(self.error_text)

    def get_current_calibration(self):
        ErrorWindow(self.error_text)

    def set_fh_time_profile(self):
        ErrorWindow(self.error_text)

    def set_fh_temp_profile(self):
        ErrorWindow(self.error_text)

    def arm_fast_heat(self):
        ErrorWindow(self.error_text)

    def run_fast_heat(self):
        ErrorWindow(self.error_text)


import logging
import time
import uldaq as ul

# TODO: add an abstract class for device + add a mock device for testing


class DaqParams:
    def __init__(self):
        # init class with default daq settings
        self.interface_type = ul.InterfaceType.ANY  # USB = 1; BLUETOOTH = 2; ETHERNET = 4; ANY = 7 from https://www.mccdaq.com/PDFs/Manuals/UL-Linux/python/api.html#uldaq.InterfaceType
        self.connection_code = -1

    def __str__(self):
        return str(vars(self))


class DaqDeviceHandler:
    def __init__(self, params: DaqParams):
        self._params = params
        self._init_daq_device()

    def _init_daq_device(self):
        # Looking for connected devices. We expect only one device for now
        # If no devices connected, raise an error
        devices = ul.get_daq_device_inventory(self._params.interface_type, 1)
        if not devices:
            error_str = "No DAQ devices found."
            logging.error("DAQ DEVICE: ERROR. {}".format(error_str))
            raise RuntimeError(error_str)

        # by default connecting only to the first DAQBoard with index 0
        self._daq_device = ul.DaqDevice(devices[0])

    def __enter__(self):
        # self.try_connect()
        return self

    def __exit__(self, exe_type, exe_value, exe_traceback):
        self.quit()

    def get_descriptor(self) -> ul.DaqDeviceDescriptor:
        return self._daq_device.get_descriptor()

    def is_connected(self) -> bool:
        return self._daq_device.is_connected()

    def connect(self):
        descriptor = self.get_descriptor()
        logging.info("DAQ DEVICE: Connecting to {} - please wait...".format(descriptor.dev_string))
        # For Ethernet devices using a connection_code other than the default
        # value of zero, change the line below to enter the desired code.
        self._daq_device.connect(connection_code=self._params.connection_code)
        if self._daq_device.is_connected():
            logging.info("DAQ DEVICE: DAQ device has been successfully connected.")
        else:
            logging.warning("DAQ DEVICE: WARNING. DAQ device hasn't been connected.")

    def try_connect(self, timeout: int = 60, sleep_time: int = 1):
        for _ in range(timeout):
            if not self.is_connected():
                self.connect()
            time.sleep(sleep_time)
            if self.is_connected():
                return
        raise TimeoutError("DAQ DEVICE: Connection timed out.")

    def disconnect(self):
        self._daq_device.disconnect()
        logging.info("DAQ DEVICE: DAQ device has been disconnected.")

    def release(self):
        self._daq_device.release()
        logging.info("DAQ DEVICE: DAQ device has been released.")

    def reset(self):
        self._daq_device.reset()
        logging.info("DAQ DEVICE: DAQ device has been reset.")

    def quit(self):
        ## AM: doesn't seems to work..
        if self.is_connected():
            self.disconnect()
        self.release()
        logging.info("DAQ DEVICE: DAQ device has been disconnected and released.")

    def get(self) -> ul.DaqDevice:
        return self._daq_device

    def get_ai_device(self) -> ul.AiDevice:
        return self._daq_device.get_ai_device()

    def get_ao_device(self) -> ul.AoDevice:
        return self._daq_device.get_ao_device()



import logging
from typing import Tuple
import uldaq as ul


class AoParams:
    def __init__(self):
        self.sample_rate = -1  # Hz
        self.range_id = -1
        self.low_channel = -1
        self.high_channel = -1
        self.scan_flags = ul.AOutScanFlag.DEFAULT  # 0
        self.options = ul.ScanOption.CONTINUOUS  # 8

    def __str__(self):
        return str(vars(self))


class AoDeviceHandler:
    """Wraps the analog-output uldaq.AoDevice."""

    def __init__(self, ao_device_from_daq: ul.AoDevice,
                 params: AoParams):
        """Initializes AO device and all parameters.

        Args:
            ao_device_from_daq: uldaq.AoDevice obtained from uldaq.DaqDevice.
            params: An AoParams instance, containing all needed analog-output parameters parsed from JSON.
            
        Raises:
            RuntimeError if the DAQ device doesn't support analog output or hardware paced analog output.
        """
        self._ao_device = ao_device_from_daq
        self._check_device()
        self._params = params

    def _check_device(self):
        if self._ao_device is None:
            error_str = "Error. DAQ device doesn't support analog output."
            logging.error(error_str)
            raise RuntimeError(error_str)

        info = self._ao_device.get_info()
        if not info.has_pacer():
            error_str = "Error. DAQ device doesn't support hardware paced analog output."
            logging.error(error_str)
            raise RuntimeError(error_str)

    def get(self) -> ul.AoDevice:
        """Provides explicit access to the uldaq.AoDevice."""
        return self._ao_device

    def stop(self):
        self._ao_device.scan_stop()

    def status(self) -> Tuple[ul.ScanStatus, ul.TransferStatus]:
        return self._ao_device.get_scan_status()

    # returns actual output scan rate
    def scan(self, ao_buffer) -> float:
        analog_range = ul.Range(self._params.range_id)
        samples_per_channel = int((len(ao_buffer) / (self._params.high_channel - self._params.low_channel + 1)))
        return self._ao_device.a_out_scan(self._params.low_channel, self._params.high_channel,
                                          analog_range, samples_per_channel,
                                          self._params.sample_rate, self._params.options, 
                                          self._params.scan_flags, ao_buffer)
    # set voltage profile to selected channel
    def iso_mode(self, ao_channel, voltage) -> float:
        analog_range = ul.Range(self._params.range_id)
        return self._ao_device.a_out(ao_channel,
                                    analog_range, 
                                    self._params.scan_flags,
                                    voltage)


from silx.gui import icons, qt

from pioner.front.procFastHeatWidget import procFastHeatWidget
from pioner.front.resultsDataWidget import resultsDataWidget
from pioner.front.SetProg_widget import *

class mainWindowUi(qt.QWidget):
    def __init__(self, parent=None):
        super(mainWindowUi, self).__init__(parent)
        self.setWindowTitle("Nanocontol")
        self.setMinimumHeight(800)

        mainLayout = qt.QHBoxLayout()
        self.setLayout(mainLayout)

        short_label_width = 55
        short_button_width = 60
        long_label_width = 60
        button_height = 20
        short_line_input_width = 60
        font = qt.QFont()
        ####### Left control panel
        ########################################
        lcp_width = 200

        leftLayout = qt.QVBoxLayout()
        mainLayout.addLayout(leftLayout)

        ####### System Group Box
        ########################################
        systemBox = qt.QGroupBox("System")
        leftLayout.addWidget(systemBox, 0)
        lout_0 = qt.QVBoxLayout()
        systemBox.setLayout(lout_0)

        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        self.sysOnButton = qt.QPushButton(" ON ")
        font.setBold(True)
        self.sysOnButton.setFont(font)
        self.sysOnButton.setMinimumHeight(23)
        lout_1.addWidget(self.sysOnButton, 1)
        self.sysOffButton = qt.QPushButton("OFF")
        font.setBold(True)
        self.sysOffButton.setFont(font)
        self.sysOffButton.setMinimumHeight(23)
        lout_1.addWidget(self.sysOffButton, 1)
        self.sysSetupButton = qt.QToolButton()
        self.sysSetupButton.setToolTip("Device settings")
        self.sysSetupButton.setIcon(icons.getQIcon('item-object'))
        lout_1.addWidget(self.sysSetupButton, 0)
        lout_1.setSpacing(1)

        self.sysNoHardware = qt.QCheckBox(" run without hardware")
        lout_0.addWidget(self.sysNoHardware)

        
        # ####### Experiment Group Box
        # ########################################
        self.experimentBox = qt.QGroupBox("Experiment")
        leftLayout.addWidget(self.experimentBox, 0)
        lout_0 = qt.QVBoxLayout()
        self.experimentBox.setLayout(lout_0)

        lout_0.addStretch()
        hline = qt.QFrame()
        hline.setFrameShape(qt.QFrame.HLine)
        hline.setStyleSheet("color: rgb(220, 220, 220);")
        lout_0.addWidget(hline)
        lout_0.addStretch()

        lout_1 = qt.QVBoxLayout()
        lout_0.addLayout(lout_1)
        lout_1.setSpacing(1)
        lout_1.addWidget(qt.QLabel("Data path"))
        lout_2 = qt.QHBoxLayout()
        lout_2.setSpacing(0)
        lout_1.addLayout(lout_2)
        self.sysDataPathInput = qt.QLineEdit()
        self.sysDataPathInput.setMinimumWidth(150)
        #self.sysDataPathInput.setFrame(False)
        lout_2.addWidget(self.sysDataPathInput)
        self.sysDataPathButton = qt.QToolButton()
        self.sysDataPathButton.setToolTip("Browse data path")
        self.sysDataPathButton.setIcon(icons.getQIcon('document-open'))
        lout_2.addWidget(self.sysDataPathButton)
        
        lout_1 = qt.QVBoxLayout()
        lout_0.addLayout(lout_1)
        lout_1.setSpacing(3)
        lout_1.addWidget(qt.QLabel("Calibration path"))
        lout_2 = qt.QHBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2.setSpacing(0)
        self.calibPathInput = qt.QLineEdit()
        #self.calibPathInput.setFrame(False)
        lout_2.addWidget(self.calibPathInput)
        self.calibPathButton = qt.QToolButton()
        self.calibPathButton.setToolTip("Browse calibration path")
        self.calibPathButton.setIcon(icons.getQIcon('document-open'))
        lout_2.addWidget(self.calibPathButton)
        lout_3 = qt.QHBoxLayout()
        lout_3.setSpacing(1)
        lout_1.addLayout(lout_3)
        lout_3.addStretch()
        self.calibViewButton = qt.QPushButton("View")
        self.calibViewButton.setFixedWidth(short_button_width)
        lout_3.addWidget(self.calibViewButton)
        self.calibApplyButton = qt.QPushButton("Apply")
        self.calibApplyButton.setFixedWidth(short_button_width)
        lout_3.addWidget(self.calibApplyButton)

        lout_0.addStretch()
        hline = qt.QFrame()
        hline.setFrameShape(qt.QFrame.HLine)
        hline.setStyleSheet("color: rgb(220, 220, 220);")
        lout_0.addWidget(hline)
        lout_0.addStretch()

        lout_1 = qt.QVBoxLayout()
        lout_0.addLayout(lout_1)
        lout_1.setSpacing(3)
        lout_1.addWidget(qt.QLabel("Scan"))
        lout_2 = qt.QHBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2.setSpacing(3)
        lout_2.addStretch()
        lout_2.addWidget(qt.QLabel("Sample rate:"))
        self.scanSampleRateInput = qt.QLineEdit()
        self.scanSampleRateInput.setMaximumWidth(short_line_input_width)
        #self.scanSampleRateInput.setFrame(False)
        lout_2.addWidget(self.scanSampleRateInput)
        unit = qt.QLabel("Hz")
        unit.setFixedWidth(20)
        lout_2.addWidget(unit)
        lout_3 = qt.QHBoxLayout()
        lout_1.addLayout(lout_3)
        lout_3.setSpacing(1)
        lout_3.addStretch()
        self.resetScanSampleRateButton = qt.QPushButton("Reset")
        self.resetScanSampleRateButton.setFixedWidth(short_button_width)
        lout_3.addWidget(self.resetScanSampleRateButton)
        self.applyScanSampleRateButton = qt.QPushButton("Apply")
        self.applyScanSampleRateButton.setFixedWidth(short_button_width)
        lout_3.addWidget(self.applyScanSampleRateButton)

        lout_0.addStretch()
        hline = qt.QFrame()
        hline.setFrameShape(qt.QFrame.HLine)
        hline.setStyleSheet("color: rgb(220, 220, 220);")
        lout_0.addWidget(hline)
        lout_0.addStretch()

        lout_1 = qt.QVBoxLayout()
        lout_0.addLayout(lout_1)
        lout_1.setSpacing(3)
        lout_1.addWidget(qt.QLabel("Modulation"))
        lout_2 = qt.QHBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2.setSpacing(3)
        lout_2.addStretch()
        lout_2.addWidget(qt.QLabel("Frequency:"))
        self.freqInput = qt.QLineEdit()
        self.freqInput.setMaximumWidth(short_line_input_width)
        #self.freqInput.setFrame(False)
        lout_2.addWidget(self.freqInput)
        units = qt.QLabel("Hz")
        units.setFixedWidth(20)
        lout_2.addWidget(units)
        lout_3 = qt.QHBoxLayout()
        lout_1.addLayout(lout_3)
        lout_3.setSpacing(3)
        lout_3.addStretch()
        lout_3.addWidget(qt.QLabel("Amplitude:"))
        self.amplitudeInput = qt.QLineEdit()
        self.amplitudeInput.setMaximumWidth(short_line_input_width)
        #self.amplitudeInput.setFrame(False)
        lout_3.addWidget(self.amplitudeInput)
        units = qt.QLabel("mA")
        units.setFixedWidth(20)
        lout_3.addWidget(units)
        lout_4 = qt.QHBoxLayout()
        lout_1.addLayout(lout_4)
        lout_4.setSpacing(3)
        lout_4.addStretch()
        lout_4.addWidget(qt.QLabel("Offset:"))
        self.offsetInput = qt.QLineEdit()
        self.offsetInput.setMaximumWidth(short_line_input_width)
        #self.offsetInput.setFrame(False)
        lout_4.addWidget(self.offsetInput)
        units = qt.QLabel("mA")
        units.setFixedWidth(20)
        lout_4.addWidget(units)
        lout_5 = qt.QHBoxLayout()
        lout_1.addLayout(lout_5)
        lout_5.setSpacing(1)
        lout_5.addStretch()
        self.resetModulationParamsButton = qt.QPushButton("Reset")
        self.resetModulationParamsButton.setFixedWidth(short_button_width)
        lout_5.addWidget(self.resetModulationParamsButton)
        self.applyModulationParamsButton = qt.QPushButton("Apply")
        self.applyModulationParamsButton.setFixedWidth(short_button_width)
        lout_5.addWidget(self.applyModulationParamsButton)

        # lout_0.addStretch()

        leftLayout.addStretch(2)

        # ####### Logo
        # ########################################
        lout_0 = qt.QVBoxLayout()
        leftLayout.addLayout(lout_0, 0)
        logo = qt.QLabel()
        logo.setAlignment(qt.Qt.AlignHCenter)
        pixmap = qt.QPixmap("./res/logo.png").scaledToWidth(70)
        logo.setPixmap(pixmap)
        lout_0.addWidget(logo)
        sign = qt.QLabel("Nanocontrol v.0.1")
        sign.setAlignment(qt.Qt.AlignHCenter)
        lout_0.addWidget(sign)


        # ####### Right experiment main view
        # ########################################
        rightLayout = qt.QVBoxLayout()
        mainLayout.addLayout(rightLayout, 1)

        self.mainTabWidget = qt.QTabWidget()
        self.controlTab = qt.QWidget()
        self.mainTabWidget.addTab(self.controlTab, "Control")
        self.resultTab = qt.QWidget()
        self.mainTabWidget.addTab(self.resultTab, "Result")
        self.addNewTab = qt.QWidget()
        self.mainTabWidget.addTab(self.addNewTab, "+")
        self.mainTabWidget.setTabsClosable(True)
        self.mainTabWidget.tabBar().setTabButton(0, qt.QTabBar.RightSide, None)
        self.mainTabWidget.tabBar().setTabButton(1, qt.QTabBar.RightSide, None)
        self.mainTabWidget.tabBar().setTabButton(2, qt.QTabBar.RightSide, None)
        rightLayout.addWidget(self.mainTabWidget)
        
        # Control tab
        lout_0 = qt.QVBoxLayout()
        self.controlTab.setLayout(lout_0)
        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1, 1)
        
        # Prog group box
        self.progGroupBox = qt.QGroupBox("Prog")
        lout_1.addWidget(self.progGroupBox, 0)
        lout_2 = qt.QVBoxLayout()
        lout_2.setSpacing(0)
        self.progGroupBox.setLayout(lout_2)
        
        lout_3 = qt.QHBoxLayout()
        lout_3.setSpacing(0)
        lout_2.addLayout(lout_3)
        self.experimentTimeComboBox = qt.QComboBox()
        self.experimentTimeComboBox.setFixedWidth(75)
        self.experimentTimeComboBox.addItem("Time (ms)")
        self.experimentTimeComboBox.addItem("Time (s)")
        lout_3.addWidget(self.experimentTimeComboBox)
        self.experimentTempComboBox = qt.QComboBox()
        self.experimentTempComboBox.setFixedWidth(75)
        self.experimentTempComboBox.addItem("Temp (°C)")
        self.experimentTempComboBox.addItem("Volt (V)")
        lout_3.addWidget(self.experimentTempComboBox)

        lout_3 = qt.QHBoxLayout()
        lout_3.setSpacing(0)
        lout_2.addLayout(lout_3)
        self.experimentTable = qt.QTableWidget()
        self.experimentTable.setFixedSize(150, 207)
        self.experimentTable.setFrameStyle(qt.QFrame.NoFrame)
        # self.experimentTable.setFrameStyle(qt.QFrame.StyledPanel)
        self.experimentTable.setRowCount(100)
        self.experimentTable.setColumnCount(2)
        self.experimentTable.horizontalHeader().setVisible(False)
        self.experimentTable.horizontalHeader().setDefaultSectionSize(75)
        self.experimentTable.horizontalHeader().setHighlightSections(False)
        self.experimentTable.horizontalHeader().setMinimumSectionSize(75)
        self.experimentTable.verticalHeader().setVisible(False)
        self.experimentTable.verticalHeader().setDefaultSectionSize(10)
        self.experimentTable.verticalHeader().setMinimumSectionSize(10)
        self.experimentTable.setVerticalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOff)
        self.experimentTable.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOff)
        self.experimentTable.setColumnWidth(0,75)
        self.experimentTable.setColumnWidth(1,75)
        self.experimentTable.setInputMethodHints(qt.Qt.ImhNone)
        for row in range(100):
            for col in range(2):
                item = qt.QTableWidgetItem('0')
                item.setTextAlignment(qt.Qt.AlignCenter)
                self.experimentTable.setItem(row, col, item)
        self.experimentTable.setFocusPolicy(qt.Qt.NoFocus)
        palette = self.experimentTable.palette()
        palette.setColor(qt.QPalette.Inactive, qt.QPalette.Highlight, qt.Qt.transparent)
        self.experimentTable.setPalette(palette)
        self.experimentTable.setEditTriggers(qt.QAbstractItemView.AllEditTriggers)
        self.experimentTable.resizeRowsToContents()
        lout_3.addWidget(self.experimentTable, 0)

        lout_2.addSpacing(5)

        lout_3 = qt.QHBoxLayout()
        lout_3.setSpacing(1)
        lout_2.addLayout(lout_3)
        self.loadTxtButton = qt.QPushButton("Load")
        lout_3.addWidget(self.loadTxtButton)
        self.armButton = qt.QPushButton("Arm")
        lout_3.addWidget(self.armButton)

        lout_2.addSpacing(5)

        lout_3 = qt.QHBoxLayout()
        lout_3.setSpacing(1)
        lout_2.addLayout(lout_3)
        self.stopButton = qt.QPushButton("Stop")
        lout_3.addWidget(self.stopButton)
        self.startButton = qt.QPushButton("Start")
        lout_3.addWidget(self.startButton)

        self.holdFinalValue = qt.QCheckBox(" hold final value")
        lout_2.addWidget(self.holdFinalValue)

        lout_2.addStretch()

        lout_3 = qt.QVBoxLayout()
        lout_3.setSpacing(5)
        lout_2.addLayout(lout_3)
        isoLabel = qt.QLabel("Isothermal mode")
        isoLabel.setAlignment(qt.Qt.AlignCenter)
        lout_3.addWidget(isoLabel)
        lout_4 = qt.QHBoxLayout()
        lout_4.setSpacing(3)
        lout_3.addLayout(lout_4)
        self.setComboBox = qt.QComboBox()
        self.setComboBox.addItem("Temperature")
        self.setComboBox.addItem("Voltage")
        lout_4.addWidget(self.setComboBox)
        self.setInput = qt.QLineEdit()
        self.setInput.setMaximumWidth(short_line_input_width)
        # self.setInput.setFrame(False)
        lout_4.addWidget(self.setInput)
        self.setInputUnits = qt.QLabel("°C")
        self.setInputUnits.setAlignment(qt.Qt.AlignLeft)
        self.setInputUnits.setFixedWidth(20)
        lout_4.addWidget(self.setInputUnits)
        lout_4.addStretch()
        lout_5 = qt.QHBoxLayout()
        lout_5.setSpacing(1)
        lout_3.addLayout(lout_5)
        self.unsetTempVoltButton = qt.QPushButton("Off")
        lout_5.addWidget(self.unsetTempVoltButton)
        self.setTempVoltButton = qt.QPushButton("Set")
        lout_5.addWidget(self.setTempVoltButton)

        # Signal and program plots 
        self.controlTabsWidget = qt.QTabWidget()
        self.signalsTab = qt.QWidget()
        self.controlTabsWidget.addTab(self.signalsTab, "Signals")
        self.progTab = qt.QWidget()
        self.controlTabsWidget.addTab(self.progTab, "Program")
        lout_1.addWidget(self.controlTabsWidget, 1)
        lout_2 = qt.QVBoxLayout()
        self.signalsTab.setLayout(lout_2)
        self.signalsPlot = resultsDataWidget()
        self.signalsPlot.resultPlot.setAxesMargins(left=0.005, top=0.005, right=0.005, bottom=0.005)
        self.signalsPlot.resultPlot.getYAxis().setLabel('')
        self.signalsPlot.resultPlot.getXAxis().setLabel('')
        lout_2.addWidget(self.signalsPlot)
        lout_3 = qt.QVBoxLayout()
        self.progTab.setLayout(lout_3)
        self.progPlot = resultsDataWidget()
        self.progPlot.resultPlot.getXAxis().setLabel(self.experimentTimeComboBox.currentText())
        self.progPlot.resultPlot.getYAxis().setLabel(self.experimentTempComboBox.currentText())
        lout_3.addWidget(self.progPlot)

        # Values area
        self.valuesBox = qt.QGroupBox("Values")
        lout_0.addWidget(self.valuesBox, 0)
        lout_1 = qt.QHBoxLayout()
        self.valuesBox.setLayout(lout_1)
        
        lout_1.addStretch()
        lout_1.addStretch()

        lout_2 = qt.QVBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2_1 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_1)
        labl = qt.QLabel("R htr abs:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_1.addWidget(labl)
        self.rhtrabsValueLabel = qt.QLabel(" ---")
        self.rhtrabsValueLabel.setFixedWidth(short_label_width)
        self.rhtrabsValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_1.addWidget(self.rhtrabsValueLabel)
        lout_2_2 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_2)
        labl = qt.QLabel("R htr dyn:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_2.addWidget(labl)
        self.rhtrdynValueLabel = qt.QLabel(" ---")
        self.rhtrdynValueLabel.setFixedWidth(short_label_width)
        self.rhtrdynValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_2.addWidget(self.rhtrdynValueLabel)
        lout_2_3 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_3)
        labl = qt.QLabel("U mod htr:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_3.addWidget(labl)
        self.umodhtrValueLabel = qt.QLabel(" ---")
        self.umodhtrValueLabel.setFixedWidth(short_label_width)
        self.umodhtrValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_3.addWidget(self.umodhtrValueLabel)
        lout_2_4 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_4)
        labl = qt.QLabel("I htr:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_4.addWidget(labl)
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_4.addWidget(labl)
        self.ihtrValueLabel = qt.QLabel(" ---")
        self.ihtrValueLabel.setFixedWidth(short_label_width)
        self.ihtrValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_4.addWidget(self.ihtrValueLabel)

        lout_1.addStretch()
        vline = qt.QFrame()
        vline.setFrameShape(qt.QFrame.VLine)
        vline.setStyleSheet("color: rgb(220, 220, 220);")
        lout_1.addWidget(vline)
        lout_1.addStretch()

        lout_2 = qt.QVBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2_1 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_1)
        labl = qt.QLabel("T aux:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_1.addWidget(labl)
        self.tauxValueLabel = qt.QLabel(" ---")
        self.tauxValueLabel.setFixedWidth(short_label_width)
        self.tauxValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_1.addWidget(self.tauxValueLabel)
        lout_2_2 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_2)
        labl = qt.QLabel("T tpl:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_2.addWidget(labl)
        self.ttplValueLabel = qt.QLabel(" ---")
        self.ttplValueLabel.setFixedWidth(short_label_width)
        self.ttplValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_2.addWidget(self.ttplValueLabel)
        lout_2_3 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_3)
        labl = qt.QLabel("T htr:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_3.addWidget(labl)
        self.thtrValueLabel = qt.QLabel(" ---")
        self.thtrValueLabel.setFixedWidth(short_label_width)
        self.thtrValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_3.addWidget(self.thtrValueLabel)
        lout_2_4 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_4)
        labl = qt.QLabel("T htr dyn:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_4.addWidget(labl)
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_4.addWidget(labl)
        self.thtrdynValueLabel = qt.QLabel(" ---")
        self.thtrdynValueLabel.setFixedWidth(short_label_width)
        self.thtrdynValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_4.addWidget(self.thtrdynValueLabel)
        
        lout_2 = qt.QVBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2_1 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_1)
        labl = qt.QLabel("T-error:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_1.addWidget(labl)
        self.terrorValueLabel = qt.QLabel(" --")
        self.terrorValueLabel.setFixedWidth(30)
        self.terrorValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_1.addWidget(self.terrorValueLabel)
        lout_2_2 = qt.QVBoxLayout()
        lout_2.addLayout(lout_2_2)
        lout_2_2.setSpacing(1)
        lout_2_2.setAlignment(qt.Qt.AlignCenter)
        self.terror0Button = qt.QPushButton("> 0 <")
        self.terror0Button.setFixedWidth(short_button_width)
        lout_2_2.addWidget(self.terror0Button)
        self.tresetButton = qt.QPushButton("reset")
        self.tresetButton.setFixedWidth(short_button_width)
        lout_2_2.addWidget(self.tresetButton)
        lout_2.addStretch()

        lout_1.addStretch()
        vline = qt.QFrame()
        vline.setFrameShape(qt.QFrame.VLine)
        vline.setStyleSheet("color: rgb(220, 220, 220);")
        lout_1.addWidget(vline)
        lout_1.addStretch()

        lout_2 = qt.QVBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2_1 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_1)
        labl = qt.QLabel("Frequency:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_1.addWidget(labl)
        self.frequencyValueLabel = qt.QLabel(" ---")
        self.frequencyValueLabel.setFixedWidth(short_label_width)
        self.frequencyValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_1.addWidget(self.frequencyValueLabel)
        lout_2_2 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_2)
        labl = qt.QLabel("Amplitude:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_2.addWidget(labl)
        self.amplitudeValueLabel = qt.QLabel(" ---")
        self.amplitudeValueLabel.setFixedWidth(short_label_width)
        self.amplitudeValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_2.addWidget(self.amplitudeValueLabel)
        lout_2_3 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_3)
        labl = qt.QLabel("Offset:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_3.addWidget(labl)
        self.offsetValueLabel = qt.QLabel(" ---")
        self.offsetValueLabel.setFixedWidth(short_label_width)
        self.offsetValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_3.addWidget(self.offsetValueLabel)
        lout_2_4 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_4)
        labl = qt.QLabel("Power:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_4.addWidget(labl)
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_4.addWidget(labl)
        self.powerValueLabel = qt.QLabel(" ---")
        self.powerValueLabel.setFixedWidth(short_label_width)
        self.powerValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_4.addWidget(self.powerValueLabel)

        lout_2 = qt.QVBoxLayout()
        lout_1.addLayout(lout_2)
        lout_2_1 = qt.QHBoxLayout()
        lout_2.addLayout(lout_2_1)
        labl = qt.QLabel("Phase:")
        labl.setAlignment(qt.Qt.AlignRight)
        lout_2_1.addWidget(labl)
        self.phaseValueLabel = qt.QLabel(" --")
        self.phaseValueLabel.setFixedWidth(30)
        self.phaseValueLabel.setAlignment(qt.Qt.AlignLeft)
        lout_2_1.addWidget(self.phaseValueLabel)
        lout_2_2 = qt.QVBoxLayout()
        lout_2.addLayout(lout_2_2)
        lout_2_2.setSpacing(1)
        lout_2_2.setAlignment(qt.Qt.AlignCenter)
        self.phase0Button = qt.QPushButton("> 0 <")
        self.phase0Button.setFixedWidth(short_button_width)
        lout_2_2.addWidget(self.phase0Button)
        self.phaseResetButton = qt.QPushButton("reset")
        self.phaseResetButton.setFixedWidth(short_button_width)
        lout_2_2.addWidget(self.phaseResetButton)
        lout_2.addStretch()

        lout_1.addStretch()
        lout_1.addStretch()

        # results tab
        lout_0 = qt.QHBoxLayout()
        self.resultTab.setLayout(lout_0)
        self.resultsDataWidget = resultsDataWidget(parent=self)
        self.resultsDataWidget.resultPlot.getXAxis().setLabel('Time (ms)')
        self.resultsDataWidget.resultPlot.getYAxis().setLabel('Temp (°C)')
        lout_0.addWidget(self.resultsDataWidget)

        # ########################################################

        
        ####### Status bar
        ########################################        
        lout_0 = qt.QHBoxLayout()
        rightLayout.addLayout(lout_0)
        lout_0.setSpacing(1)
        lout_0.addWidget(qt.QLabel("Hardware:"))
        self.hardware_label = qt.QLabel(' ---')
        self.hardware_label.setFixedWidth(80)
        lout_0.addWidget(self.hardware_label)
        lout_0.addWidget(qt.QLabel("Status:"))
        self.status_label = qt.QLabel(' ---')
        self.status_label.setFixedWidth(80)
        lout_0.addWidget(self.status_label)
        lout_0.addStretch()
        lout_0.addWidget(qt.QLabel("Progress:"))
        self.progressBar = qt.QProgressBar()
        lout_0.addWidget(self.progressBar,1)
        
        ####### Filtering input
        #######################################
        class TableDelegate(qt.QItemDelegate):
            def __init__(self, parent):
                qt.QItemDelegate.__init__(self, parent)
            def createEditor(self, parent, option, index):
                spinbox = qt.QDoubleSpinBox(parent)
                spinbox.setMaximum(100000)
                return spinbox

        self.experimentTable.setItemDelegate(TableDelegate(self.experimentTable))
        self.experimentTable.setItemDelegate(TableDelegate(self.experimentTable))

        float_validator_1000 = qt.QRegExpValidator(qt.QRegExp("^[0-9]{1,3}\.[0-9]{1,2}$"))
        float_validator_100 = qt.QRegExpValidator(qt.QRegExp("^[0-9]{1,2}\.[0-9]{1,2}$"))
        int_validator_5 = qt.QRegExpValidator(qt.QRegExp("^[0-9]{1,5}$"))

        self.scanSampleRateInput.setValidator(int_validator_5)
        self.freqInput.setValidator(float_validator_100)
        self.amplitudeInput.setValidator(float_validator_100)
        self.offsetInput.setValidator(float_validator_100)
        self.setInput.setValidator(float_validator_1000)

        ####### Set items disabled
        ####################################### 
        # [item.setEnabled(False) for item in [self.experimentBox, self.controlTab]]

        self.setComboBox.currentTextChanged.connect(self.setComboBox_changed)
        
        ####### Add/delete tabs on mainTabWidget
        #######################################         
        self.mainTabWidget.currentChanged.connect(self.add_tab_to_mainTabWidget)
        self.mainTabWidget.tabCloseRequested.connect(self.close_tab_in_mainTabWidget)

    def setComboBox_changed(self):
        text = self.setComboBox.currentText()
        if text == "Temp":
            self.setInputUnits.setText("°C")
        if text == "Volt":
            self.setInputUnits.setText("V")
    
    def add_tab_to_mainTabWidget(self, i):
        if self.mainTabWidget.tabText(i) == "+":
            tab_types = ("Process fast heating", 
                        "Process slow heating", 
                        "Process with custom workflow",
                        "Advanced process control")
            tab_type, ok = qt.QInputDialog.getItem(self, "Choose type:", 
                "New tab", tab_types, 0, False)
            if ok and tab_type:
                self.newTab = qt.QWidget()
                self.newTab.layout = qt.QVBoxLayout()
                self.mainTabWidget.insertTab(i, self.newTab, tab_type)   
                self.mainTabWidget.setCurrentIndex(i)

                if tab_type=="Process fast heating":
                    self.newTabWidget = procFastHeatWidget(self)
                    self.newTab.layout.addWidget(self.newTabWidget)
                if tab_type == "Advanced process control":
                    self.newTabWidget = SetProg(self)
                    self.newTab.layout.addWidget(self.newTabWidget)
                self.newTab.setLayout(self.newTab.layout)

    def close_tab_in_mainTabWidget(self, i):
        self.mainTabWidget.setCurrentIndex(i-1)
        self.mainTabWidget.removeTab(i)
        

if __name__ == "__main__":
    import sys
    app = qt.QApplication(sys.argv)
    example = mainWindowUi()
    example.show()
    sys.exit(app.exec())



import logging
from ctypes import Array
from typing import Tuple
import uldaq as ul


class AiParams:
    def __init__(self):
        self.sample_rate = -1  # Hz
        self.range_id = -1
        self.low_channel = -1
        self.high_channel = -1
        self.input_mode = ul.AiInputMode.SINGLE_ENDED  # 2
        self.scan_flags = ul.AInScanFlag.DEFAULT  # 0
        self.options = ul.ScanOption.CONTINUOUS  # 8

    def __str__(self):
        return str(vars(self))


class AiDeviceHandler:
    """Wraps the analog-input uldaq.AiDevice."""

    def __init__(self, ai_device_from_daq: ul.AiDevice,
                 params: AiParams):
        """Initializes AI device and all parameters.

        Args:
            ai_device_from_daq: uldaq.AiDevice obtained from uldaq.DaqDevice.
            params: An AiParams instance, containing all needed analog-input parameters parsed from JSON.
        Raises:
            RuntimeError if the DAQ device doesn't support analog input or hardware paced analog input.
        """
        self._ai_device = ai_device_from_daq
        self._params = params
        self._check_device()
        self._init_buffer()

    def _check_device(self):
        if self._ai_device is None:
            error_str = "Error. DAQ device doesn't support analog input."
            logging.error(error_str)
            raise RuntimeError(error_str)

        info = self._ai_device.get_info()
        if not info.has_pacer():
            error_str = "Error. DAQ device doesn't support hardware paced analog input."
            logging.error(error_str)
            raise RuntimeError(error_str)

        if info.get_num_chans_by_mode(ul.AiInputMode.SINGLE_ENDED) <= 0:
            self._params.input_mode = ul.AiInputMode.DIFFERENTIAL

    def _init_buffer(self):
        channel_count = self._params.high_channel - self._params.low_channel + 1
        self._buffer = ul.create_float_buffer(channel_count, self._params.sample_rate)

    def get(self) -> ul.AiDevice:
        """Provides explicit access to the uldaq.AiDevice."""
        return self._ai_device                                   

    def get_buffer(self) -> Array[float]:
        """Returns an array of double precision floating point sample values."""
        return self._buffer

    def stop(self):
        self._ai_device.scan_stop()
  
    def status(self) -> Tuple[ul.ScanStatus, ul.TransferStatus]:
        return self._ai_device.get_scan_status()

    # returns actual input scan rate
    def scan(self) -> float:
        analog_range = ul.Range(self._params.range_id)
        return self._ai_device.a_in_scan(self._params.low_channel, self._params.high_channel, 
                                         self._params.input_mode, analog_range, self._params.sample_rate,
                                         self._params.sample_rate, self._params.options, 
                                         self._params.scan_flags, self._buffer)



import sys
from scipy import interpolate
import numpy as np
import matplotlib.pyplot as plt
import time

from shared.constants import *
from shared.settings import BackSettings
from shared.calibration import Calibration
from shared.utils import temperature_to_voltage
from back.daq_device import DaqDeviceHandler
from back.fastheat import FastHeat

def debug_time_temp_table():
    time_temp_table = {
        'time'.TIME: [0, 50, 450, 550, 950, 1000],
        'temperature': [0, 0, 5, 5, 0, 0]
    }

    # for not to truncate printing numpy arrays
    np.set_printoptions(threshold=sys.maxsize)

    time = time_temp_table['time']
    # print("time: \n{}".format(time))

    temp = time_temp_table['temperature']
    # print("temp: \n{}".format(temp))

    interpolation = interpolate.interp1d(x=time, y=temp, kind='linear')

    points_num = time[-1] + 1  # to get "time[-1]" intervals !!
    time_program_points = np.linspace(time[0], time[-1], points_num)
    # print("time upd: \n{}".format(time_program_points))

    temp_program_points = interpolation(time_program_points)
    # print("temp upd: \n{}".format(temp_program_points))

    calibration = Calibration()
    calibration.read('./calibration.json')
    # print(calibration.safe_voltage, calibration.max_temp, calibration.min_temp)

    volt_program_points = temperature_to_voltage(temp_program_points, calibration)
    # print("voltage: \n{}".format(volt_program_points))

def debug_fast_heat():

    calibration = Calibration()
    calibration.read(DEFAULT_CALIBRATION_FILE_REL_PATH)
    settings = BackSettings(SETTINGS_FILE_REL_PATH)
    daq_device_handler = DaqDeviceHandler(settings.daq_params)
    print("DAQ settings: ", settings.daq_params)
    print("AI settings: ", settings.ai_params)
    print("AO settings: ", settings.ao_params)
    print("Calibration: ", calibration.comment)
    
    daq_device_handler.try_connect()
    descriptor = daq_device_handler.get_descriptor()
    print("TANGO: Product info: {}".format(descriptor.dev_string))
    
    time_temp_table = {'ch0':{'time':[0, 3000], 
                                'temp':[0.1, 0.1]},
                        'ch1':{'time':[0, 100, 1100, 1900, 2900, 3000], 
                                'temp':[0, 0, 1, 1, 0, 0]},
                    }
    # fig, ax1 = plt.subplots()
    # ax1.plot(time_temp_table['ch1']['time'], time_temp_table['ch1']['temp'])
    # plt.show()

    fh = FastHeat(daq_device_handler, settings,
                    time_temp_table, calibration,
                    ai_channels = [0,1,3,4,5],
                    FAST_HEAT_CUSTOM_FLAG=False)
    print("LOG: Fast heating armed.")

    if fh.is_armed():
        print("LOG: Fast heating started.")
        t1 = time.time()
        fh.run()
        print("LOG: Fast heating finished. Took {} s.".format(time.time()-t1))

    daq_device_handler.disconnect()


if __name__ == "__main__":
    # debug_time_temp_table()
    debug_fast_heat()


from os import path

# Common constants
# =================================================================================
MAX_SCAN_SAMPLE_RATE = 1000000
JSON_EXTENSION = "json"
H5_EXTENSION = "h5"

ROOT_PATH = path.dirname(path.dirname(path.realpath(__file__)))

# Settings constants
# =================================================================================
SETTINGS_FOLDER = "settings"

SETTINGS_FOLDER_REL_PATH = path.join("./", SETTINGS_FOLDER)     #./ for user settings files to use folder from where package was called
SETTINGS_FILE = "settings.json"
SETTINGS_FILE_REL_PATH = path.join(SETTINGS_FOLDER_REL_PATH, SETTINGS_FILE)

DEFAULT_SETTINGS_FOLDER_REL_PATH = path.join(ROOT_PATH, SETTINGS_FOLDER)    # use package installation directory
DEFAULT_SETTINGS_FILE = "default_settings.json"
DEFAULT_SETTINGS_FILE_REL_PATH = path.join(DEFAULT_SETTINGS_FOLDER_REL_PATH, DEFAULT_SETTINGS_FILE)

# JSON fields backend
DAQ_SETTINGS_FIELD = "DAQ settings"

DAQ_FIELD = "DAQ"
AI_FIELD = "AI"
AO_FIELD = "AO"

INTERFACE_TYPE_FIELD = "InterfaceType"
CONNECTION_CODE_FIELD = "ConnectionCode"

RANGE_ID_FIELD = "RangeId"
LOW_CHANNEL_FIELD = "LowChannel"
HIGH_CHANNEL_FIELD = "HighChannel"
INPUT_MODE_FIELD = "InputMode"
SCAN_FLAGS_FIELD = "ScanFlags"

# JSON fields server
SERVER_SETTINGS_FIELD = "Server settings"

TANGO_FIELD = "Tango"
TANGO_HOST_FIELD = "Tango host"
DEVICE_PROXY_FIELD = "Device proxy"

HTTP_FIELD = "HTTP"
HTTP_HOST = "HTTP host"

# JSON fields experiment 
EXPERIMENT_SETTINGS_FIELD = "Experiment settings"

PATHS_FIELD = "Paths"
CALIB_PATH_FIELD = "Calibration path"
DATA_PATH_FIELD = "Data path"

SCAN_FIELD = "Scan"
SAMPLE_RATE_FIELD = "Sample rate"

MODULATION_FIELD = "Modulation"
FREQUENCY_FIELD = "Frequency"
AMPLITUDE_FIELD = "Amplitude"
OFFSET_FIELD = "Offset"

# Raw data constants
# =================================================================================
DATA_FOLDER = "data"
DATA_FOLDER_REL_PATH = path.join("./", DATA_FOLDER)
RAW_DATA_FOLDER = "raw_data"
RAW_DATA_FOLDER_REL_PATH = path.join(DATA_FOLDER_REL_PATH, RAW_DATA_FOLDER)
RAW_DATA_FILE = "raw_data.h5"
RAW_DATA_FILE_REL_PATH = path.join(RAW_DATA_FOLDER_REL_PATH, RAW_DATA_FILE)

RAW_DATA_BUFFER_FILE_PREFIX = "raw_data_buffer_"
RAW_DATA_BUFFER_FILE_FORMAT = "raw_data_buffer_{}.h5"
BUFFER_DUMMY_1 = "raw_data_dummy_1.h5"
BUFFER_DUMMY_2 = "raw_data_dummy_2.h5"

EXP_DATA_FILE = "exp_data.h5"
EXP_DATA_FILE_REL_PATH = path.join(DATA_FOLDER_REL_PATH, EXP_DATA_FILE)

# Logs constants
# =================================================================================
LOGS_FOLDER = "logs"
LOGS_FOLDER_REL_PATH = path.abspath(path.join("./", LOGS_FOLDER))
NANOCONTROL_LOG_FILE = "nanocontrol.log"
NANOCONTROL_LOG_FILE_REL_PATH = path.join(LOGS_FOLDER_REL_PATH, NANOCONTROL_LOG_FILE)

# Calibration constants
# =================================================================================
CALIBRATION_FILE = "calibration.json"
CALIBRATION_FILE_REL_PATH = path.join(SETTINGS_FOLDER_REL_PATH, CALIBRATION_FILE)
DEFAULT_CALIBRATION_FILE = "default_calibration.json"
DEFAULT_CALIBRATION_FILE_REL_PATH = path.join(DEFAULT_SETTINGS_FOLDER_REL_PATH, DEFAULT_CALIBRATION_FILE)

INFO_FIELD = "Info"

CALIBRATION_COEFFS_FIELD = "Calibration coeff"
U_TPL_FIELD = "Utpl"
T_TPL_FIELD = "Ttpl"
T_HTR_FIELD = "Thtr"
T_HTRD_FIELD = "Thtrd"
U_HTR_FIELD = "Uhtr"
I_HTR_FIELD = "Ihtr"
T_HEATER_FIELD = "Theater"
AMPLITUDE_CORRECTION_FIELD = "Amplitude correction"
R_HEATER_FIELD = "R heater"
R_GUARD_FIELD = "R guard"
HEATER_SAFE_VOLTAGE_FIELD = "Heater safe voltage"
CORR_FIELD = "corr"
# =================================================================================


if __name__=="__main__":
    print(ROOT_PATH)


import logging

from shared.constants import *
from shared.settings import BackSettings
from shared.calibration import Calibration
from shared.utils import temperature_to_voltage
from back.daq_device import DaqDeviceHandler
from back.experiment_manager import ExperimentManager



class IsoMode:
    # receives chan_temp_volt as dict :
    # {"ch0": {"temp":float} }
    # voltage can be used instead of temperature ("volt" instead of "temp")
    # if "volt" - no calibration applied
    # if "temp" - calibration applied

    def __init__(self, daq_device_handler: DaqDeviceHandler,
                 settings: BackSettings,
                 chan_temp_volt: dict,
                 calibration: Calibration):
        self._daq_device_handler = daq_device_handler
        self._settings = settings
        self._chan_temp_volt = chan_temp_volt
        self._calibration = calibration

        self._channel = None
        self._voltage = None

    def arm(self) -> [int, float]:
        self._channel = int(list(self._chan_temp_volt.keys())[0].replace('ch', ''))
        key = list(list(self._chan_temp_volt.values())[0].keys())[0]
        self._voltage = float(list(list(self._chan_temp_volt.values())[0].values())[0])
        if key=='temp':
            self._voltage = float(temperature_to_voltage([self._voltage], self._calibration))
        return self._channel, self._voltage

    def is_armed(self) -> bool:
        return bool(self._channel is not None and self._voltage is not None)

    def run(self, do_ai:bool):
        with ExperimentManager(self._daq_device_handler,
                               self._settings) as self.em:
            self.em.ao_set(self._channel, self._voltage)
            if do_ai:
                self.em.ai_continuous([0,1,2,3,4,5], do_save_data=False)

    def ai_stop(self):
        self.em.ai_continuous_stop()


if __name__ == '__main__':

    settings = BackSettings(SETTINGS_FILE_REL_PATH)
    chan_temp_volt = {'ch2':{'volt':0},
                        }
    calibration = Calibration()

    daq_params = settings.daq_params
    daq_device_handler = DaqDeviceHandler(daq_params)
    daq_device_handler.try_connect()

    sm = IsoMode(daq_device_handler, settings, chan_temp_volt, calibration)
    sm.is_armed()
    sm.arm()
    sm.run(do_ai=True)

    daq_device_handler.disconnect()